@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -110,15 +110,13 @@ cm_RankServer(cm_server_t * tsp)
     return code;
 }
 
-void 
+void
 cm_PingServer(cm_server_t *tsp)
 {
     long code;
     int wasDown = 0;
     cm_conn_t *connp;
     struct rx_connection * rxconnp;
-    long secs;
-    long usecs;
     Capabilities caps = {0, 0};
     char hoststr[16];
     cm_req_t req;
@@ -130,18 +128,18 @@ cm_PingServer(cm_server_t *tsp)
 	lock_ObtainMutex(&tsp->mx);
 	tsp->waitCount--;
 	if (tsp->waitCount == 0)
-	    tsp->flags &= ~CM_SERVERFLAG_PINGING;
-	else 
+	    _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
+	else
 	    osi_Wakeup((LONG_PTR)tsp);
 	lock_ReleaseMutex(&tsp->mx);
 	return;
     }
-    tsp->flags |= CM_SERVERFLAG_PINGING;
+    _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PINGING);
     wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
     afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
     lock_ReleaseMutex(&tsp->mx);
 
-    code = cm_ConnByServer(tsp, cm_rootUserp, &connp);
+    code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &connp);
     if (code == 0) {
 	/* now call the appropriate ping call.  Drop the timeout if
 	* the server is known to be down, so that we don't waste a
@@ -149,7 +147,7 @@ cm_PingServer(cm_server_t *tsp)
 	*/
 
 	osi_Log4(afsd_logp, "cm_PingServer server %s (%s) was %s with caps 0x%x",
-		  osi_LogSaveString(afsd_logp, hoststr), 
+		  osi_LogSaveString(afsd_logp, hoststr),
 		  tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
 		  wasDown ? "down" : "up",
 		  tsp->capabilities);
@@ -163,8 +161,6 @@ cm_PingServer(cm_server_t *tsp)
 	else {
 	    /* file server */
 	    code = RXAFS_GetCapabilities(rxconnp, &caps);
-	    if (code == RXGEN_OPCODE)
-		code = RXAFS_GetTime(rxconnp, &secs, &usecs);
 	}
 	if (wasDown)
 	    rx_SetConnDeadTime(rxconnp, ConnDeadtimeout);
@@ -173,15 +169,16 @@ cm_PingServer(cm_server_t *tsp)
     }	/* got an unauthenticated connection to this server */
 
     lock_ObtainMutex(&tsp->mx);
-    if (code >= 0) {
+    if (code >= 0 || code == RXGEN_OPCODE || code == RX_CALL_BUSY) {
 	/* mark server as up */
-	tsp->flags &= ~CM_SERVERFLAG_DOWN;
+	_InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
         tsp->downTime = 0;
 
 	/* we currently handle 32-bits of capabilities */
-	if (caps.Capabilities_len > 0) {
+	if (code != RXGEN_OPCODE && code != RX_CALL_BUSY &&
+            caps.Capabilities_len > 0) {
 	    tsp->capabilities = caps.Capabilities_val[0];
-	    xdr_free(caps.Capabilities_val, caps.Capabilities_len);
+	    xdr_free((xdrproc_t) xdr_Capabilities, &caps);
 	    caps.Capabilities_len = 0;
 	    caps.Capabilities_val = 0;
 	} else {
@@ -189,7 +186,7 @@ cm_PingServer(cm_server_t *tsp)
 	}
 
 	osi_Log3(afsd_logp, "cm_PingServer server %s (%s) is up with caps 0x%x",
-		  osi_LogSaveString(afsd_logp, hoststr), 
+		  osi_LogSaveString(afsd_logp, hoststr),
 		  tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
 		  tsp->capabilities);
 
@@ -219,7 +216,7 @@ cm_PingServer(cm_server_t *tsp)
     } else {
 	/* mark server as down */
         if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
-            tsp->flags |= CM_SERVERFLAG_DOWN;
+            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_DOWN);
             tsp->downTime = time(NULL);
         }
 	if (code != VRESTARTING) {
@@ -228,7 +225,7 @@ cm_PingServer(cm_server_t *tsp)
             lock_ObtainMutex(&tsp->mx);
         }
 	osi_Log3(afsd_logp, "cm_PingServer server %s (%s) is down with caps 0x%x",
-		  osi_LogSaveString(afsd_logp, hoststr), 
+		  osi_LogSaveString(afsd_logp, hoststr),
 		  tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
 		  tsp->capabilities);
 
@@ -258,8 +255,8 @@ cm_PingServer(cm_server_t *tsp)
     }
 
     if (tsp->waitCount == 0)
-	tsp->flags &= ~CM_SERVERFLAG_PINGING;
-    else 
+	_InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
+    else
 	osi_Wakeup((LONG_PTR)tsp);
     lock_ReleaseMutex(&tsp->mx);
 }
@@ -320,9 +317,9 @@ static void cm_CheckServersSingular(afs_uint32 flags, cm_cell_t *cellp)
         if ((cellp == NULL || cellp == tsp->cellp) &&
              ((isDown && (flags & CM_FLAG_CHECKDOWNSERVERS)) ||
                (!isDown && (flags & CM_FLAG_CHECKUPSERVERS))) &&
-             ((!(flags & CM_FLAG_CHECKVLDBSERVERS) || 
+             ((!(flags & CM_FLAG_CHECKVLDBSERVERS) ||
                isVLDB && (flags & CM_FLAG_CHECKVLDBSERVERS)) &&
-              (!(flags & CM_FLAG_CHECKFILESERVERS) || 
+              (!(flags & CM_FLAG_CHECKFILESERVERS) ||
                  isFS && (flags & CM_FLAG_CHECKFILESERVERS)))) {
             doPing = 1;
         }	/* we're supposed to check this up/down server */
@@ -331,7 +328,7 @@ static void cm_CheckServersSingular(afs_uint32 flags, cm_cell_t *cellp)
         /* at this point, we've adjusted the server state, so do the ping and
          * adjust things.
          */
-        if (doPing) 
+        if (doPing)
 	    cm_PingServer(tsp);
 
         /* also, run the GC function for connections on all of the
@@ -343,16 +340,16 @@ static void cm_CheckServersSingular(afs_uint32 flags, cm_cell_t *cellp)
         cm_PutServerNoLock(tsp);
     }
     lock_ReleaseRead(&cm_serverLock);
-}       
+}
 
 static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
 {
-    /* 
-     * The goal of this function is to probe simultaneously 
-     * probe all of the up/down servers (vldb/file) as 
+    /*
+     * The goal of this function is to probe simultaneously
+     * probe all of the up/down servers (vldb/file) as
      * specified by flags in the minimum number of RPCs.
      * Effectively that means use one multi_RXAFS_GetCapabilities()
-     * followed by possibly one multi_RXAFS_GetTime() and 
+     * followed by possibly one multi_RXAFS_GetTime() and
      * one multi_VL_ProbeServer().
      *
      * To make this work we must construct the list of vldb
@@ -365,15 +362,13 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
     cm_conn_t **conns = NULL;
     struct rx_connection **rxconns = NULL;
     cm_req_t req;
-    afs_int32 i, j, nconns = 0, maxconns;
+    afs_int32 i, nconns = 0, maxconns;
     afs_int32 *conntimer, *results;
     Capabilities *caps = NULL;
     cm_server_t ** serversp, *tsp;
     afs_uint32 isDown, wasDown;
     afs_uint32 code;
-    time_t start, end, *deltas;
-    afs_int32 secs;
-    afs_int32 usecs;
+    time_t start, *deltas;
     char hoststr[16];
 
     cm_InitReq(&req);
@@ -391,12 +386,12 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
 
     memset(caps, 0, maxconns * sizeof(Capabilities));
 
-    if ((flags & CM_FLAG_CHECKFILESERVERS) || 
+    if ((flags & CM_FLAG_CHECKFILESERVERS) ||
         !(flags & (CM_FLAG_CHECKFILESERVERS|CM_FLAG_CHECKVLDBSERVERS)))
     {
         lock_ObtainRead(&cm_serverLock);
         for (nconns=0, tsp = cm_allServersp; tsp && nconns < maxconns; tsp = tsp->allNextp) {
-            if (tsp->type != CM_SERVER_FILE || 
+            if (tsp->type != CM_SERVER_FILE ||
                 tsp->cellp == NULL ||           /* SetPref only */
                 cellp && cellp != tsp->cellp)
                 continue;
@@ -416,11 +411,11 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                 continue;
             }
 
-            tsp->flags |= CM_SERVERFLAG_PINGING;
+            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PINGING);
             lock_ReleaseMutex(&tsp->mx);
 
             serversp[nconns] = tsp;
-            code = cm_ConnByServer(tsp, cm_rootUserp, &conns[nconns]);
+            code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &conns[nconns]);
             if (code) {
                 lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
@@ -447,10 +442,6 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
 
         /* Process results of servers that support RXAFS_GetCapabilities */
         for (i=0; i<nconns; i++) {
-            /* Leave the servers that did not support GetCapabilities alone */
-            if (results[i] == RXGEN_OPCODE)
-                continue;
-
             if (conntimer[i])
                 rx_SetConnDeadTime(rxconns[i], ConnDeadtimeout);
             rx_PutConnection(rxconns[i]);
@@ -462,15 +453,17 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if (results[i] >= 0)  {
+            if (results[i] >= 0 || results[i] == RXGEN_OPCODE ||
+                results[i] == RX_CALL_BUSY)  {
                 /* mark server as up */
-                tsp->flags &= ~CM_SERVERFLAG_DOWN;
+                _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
                 tsp->downTime = 0;
 
                 /* we currently handle 32-bits of capabilities */
-                if (caps[i].Capabilities_len > 0) {
+                if (results[i] != RXGEN_OPCODE && results[i] != RX_CALL_BUSY &&
+                    caps[i].Capabilities_len > 0) {
                     tsp->capabilities = caps[i].Capabilities_val[0];
-                    xdr_free(caps[i].Capabilities_val, caps[i].Capabilities_len);
+                    xdr_free((xdrproc_t) xdr_Capabilities, &caps[i]);
                     caps[i].Capabilities_len = 0;
                     caps[i].Capabilities_val = 0;
                 } else {
@@ -479,7 +472,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
 
                 afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
                 osi_Log3(afsd_logp, "cm_MultiPingServer server %s (%s) is up with caps 0x%x",
-                          osi_LogSaveString(afsd_logp, hoststr), 
+                          osi_LogSaveString(afsd_logp, hoststr),
                           tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
                           tsp->capabilities);
 
@@ -509,7 +502,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             } else {
                 /* mark server as down */
                 if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
-                    tsp->flags |= CM_SERVERFLAG_DOWN;
+                    _InterlockedOr(&tsp->flags, CM_SERVERFLAG_DOWN);
                     tsp->downTime = time(NULL);
                 }
                 if (code != VRESTARTING) {
@@ -519,7 +512,7 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                 }
                 afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
                 osi_Log3(afsd_logp, "cm_MultiPingServer server %s (%s) is down with caps 0x%x",
-                          osi_LogSaveString(afsd_logp, hoststr), 
+                          osi_LogSaveString(afsd_logp, hoststr),
                           tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
                           tsp->capabilities);
 
@@ -549,148 +542,17 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             }
 
             if (tsp->waitCount == 0)
-                tsp->flags &= ~CM_SERVERFLAG_PINGING;
-            else 
+                _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
+            else
                 osi_Wakeup((LONG_PTR)tsp);
-            
-            lock_ReleaseMutex(&tsp->mx);
 
-            cm_PutServer(tsp);
-        }
-
-        /* 
-         * At this point we have handled any responses that did not indicate
-         * that RXAFS_GetCapabilities is not supported.
-         */
-        for ( i=0, j=0; i<nconns; i++) {
-            if (results[i] == RXGEN_OPCODE) {
-                if (i != j) {
-                    conns[j] = conns[i];
-                    rxconns[j] = rxconns[i];
-                    serversp[j] = serversp[i];
-                }
-                j++;
-            }
-        }
-        nconns = j;
-
-        if (nconns) {
-            /* Perform the multi call */
-            start = time(NULL);
-            multi_Rx(rxconns,nconns)
-            {
-                secs = usecs = 0;
-                multi_RXAFS_GetTime(&secs, &usecs);
-                end = time(NULL);
-                results[multi_i]=multi_error;
-                if ((start == end) && !multi_error)
-                    deltas[multi_i] = end - secs;
-            } multi_End;
-        }
-
-        /* Process Results of servers that only support RXAFS_GetTime */
-        for (i=0; i<nconns; i++) {
-            /* Leave the servers that did not support GetCapabilities alone */
-            if (conntimer[i])
-                rx_SetConnDeadTime(rxconns[i], ConnDeadtimeout);
-            rx_PutConnection(rxconns[i]);
-            cm_PutConn(conns[i]);
-
-            tsp = serversp[i];
-            cm_GCConnections(tsp);
-
-            lock_ObtainMutex(&tsp->mx);
-            wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
-
-            if (results[i] >= 0)  {
-                /* mark server as up */
-                tsp->flags &= ~CM_SERVERFLAG_DOWN;
-                tsp->downTime = 0;
-                tsp->capabilities = 0;
-
-                afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
-                osi_Log3(afsd_logp, "cm_MultiPingServer server %s (%s) is up with caps 0x%x",
-                          osi_LogSaveString(afsd_logp, hoststr), 
-                          tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
-                          tsp->capabilities);
-
-                /* Now update the volume status if necessary */
-                if (wasDown) {
-                    cm_server_vols_t * tsrvp;
-                    cm_volume_t * volp;
-                    int i;
-
-                    for (tsrvp = tsp->vols; tsrvp; tsrvp = tsrvp->nextp) {
-                        for (i=0; i<NUM_SERVER_VOLS; i++) {
-                            if (tsrvp->ids[i] != 0) {
-                                cm_InitReq(&req);
-
-                                lock_ReleaseMutex(&tsp->mx);
-                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
-                                                         &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
-                                lock_ObtainMutex(&tsp->mx);
-                                if (code == 0) {
-                                    cm_UpdateVolumeStatus(volp, tsrvp->ids[i]);
-                                    cm_PutVolume(volp);
-                                }
-                            }
-                        }
-                    }
-                }
-            } else {
-                /* mark server as down */
-                if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
-                    tsp->flags |= CM_SERVERFLAG_DOWN;
-                    tsp->downTime = time(NULL);
-                }
-                if (code != VRESTARTING) {
-                    lock_ReleaseMutex(&tsp->mx);
-                    cm_ForceNewConnections(tsp);
-                    lock_ObtainMutex(&tsp->mx);
-                }
-                afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
-                osi_Log3(afsd_logp, "cm_MultiPingServer server %s (%s) is down with caps 0x%x",
-                          osi_LogSaveString(afsd_logp, hoststr), 
-                          tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
-                          tsp->capabilities);
-
-                /* Now update the volume status if necessary */
-                if (!wasDown) {
-                    cm_server_vols_t * tsrvp;
-                    cm_volume_t * volp;
-                    int i;
-
-                    for (tsrvp = tsp->vols; tsrvp; tsrvp = tsrvp->nextp) {
-                        for (i=0; i<NUM_SERVER_VOLS; i++) {
-                            if (tsrvp->ids[i] != 0) {
-                                cm_InitReq(&req);
-
-                                lock_ReleaseMutex(&tsp->mx);
-                                code = cm_FindVolumeByID(tsp->cellp, tsrvp->ids[i], cm_rootUserp,
-                                                         &req, CM_GETVOL_FLAG_NO_LRU_UPDATE, &volp);
-                                lock_ObtainMutex(&tsp->mx);
-                                if (code == 0) {
-                                    cm_UpdateVolumeStatus(volp, tsrvp->ids[i]);
-                                    cm_PutVolume(volp);
-                                }
-                            }
-                        }
-                    }
-                }
-            }
-
-            if (tsp->waitCount == 0)
-                tsp->flags &= ~CM_SERVERFLAG_PINGING;
-            else 
-                osi_Wakeup((LONG_PTR)tsp);
-            
             lock_ReleaseMutex(&tsp->mx);
 
             cm_PutServer(tsp);
         }
     }
 
-    if ((flags & CM_FLAG_CHECKVLDBSERVERS) || 
+    if ((flags & CM_FLAG_CHECKVLDBSERVERS) ||
         !(flags & (CM_FLAG_CHECKFILESERVERS|CM_FLAG_CHECKVLDBSERVERS)))
     {
         lock_ObtainRead(&cm_serverLock);
@@ -715,11 +577,11 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                 continue;
             }
 
-            tsp->flags |= CM_SERVERFLAG_PINGING;
+            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_PINGING);
             lock_ReleaseMutex(&tsp->mx);
 
             serversp[nconns] = tsp;
-            code = cm_ConnByServer(tsp, cm_rootUserp, &conns[nconns]);
+            code = cm_ConnByServer(tsp, cm_rootUserp, FALSE, &conns[nconns]);
             if (code) {
                 lock_ObtainRead(&cm_serverLock);
                 cm_PutServerNoLock(tsp);
@@ -758,21 +620,21 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
             lock_ObtainMutex(&tsp->mx);
             wasDown = tsp->flags & CM_SERVERFLAG_DOWN;
 
-            if (results[i] >= 0)  {
+            if (results[i] >= 0 || results[i] == RX_CALL_BUSY)  {
                 /* mark server as up */
-                tsp->flags &= ~CM_SERVERFLAG_DOWN;
+                _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_DOWN);
                 tsp->downTime = 0;
                 tsp->capabilities = 0;
 
                 afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
                 osi_Log3(afsd_logp, "cm_MultiPingServer server %s (%s) is up with caps 0x%x",
-                          osi_LogSaveString(afsd_logp, hoststr), 
+                          osi_LogSaveString(afsd_logp, hoststr),
                           tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
                           tsp->capabilities);
             } else {
                 /* mark server as down */
                 if (!(tsp->flags & CM_SERVERFLAG_DOWN)) {
-                    tsp->flags |= CM_SERVERFLAG_DOWN;
+                    _InterlockedOr(&tsp->flags, CM_SERVERFLAG_DOWN);
                     tsp->downTime = time(NULL);
                 }
                 if (code != VRESTARTING) {
@@ -782,16 +644,16 @@ static void cm_CheckServersMulti(afs_uint32 flags, cm_cell_t *cellp)
                 }
                 afs_inet_ntoa_r(tsp->addr.sin_addr.S_un.S_addr, hoststr);
                 osi_Log3(afsd_logp, "cm_MultiPingServer server %s (%s) is down with caps 0x%x",
-                          osi_LogSaveString(afsd_logp, hoststr), 
+                          osi_LogSaveString(afsd_logp, hoststr),
                           tsp->type == CM_SERVER_VLDB ? "vldb" : "file",
                           tsp->capabilities);
             }
 
             if (tsp->waitCount == 0)
-                tsp->flags &= ~CM_SERVERFLAG_PINGING;
-            else 
+                _InterlockedAnd(&tsp->flags, ~CM_SERVERFLAG_PINGING);
+            else
                 osi_Wakeup((LONG_PTR)tsp);
-            
+
             lock_ReleaseMutex(&tsp->mx);
 
             cm_PutServer(tsp);
@@ -832,7 +694,7 @@ void cm_CheckServers(afs_uint32 flags, cm_cell_t *cellp)
 void cm_InitServer(void)
 {
     static osi_once_t once;
-        
+
     if (osi_Once(&once)) {
         lock_InitializeRWLock(&cm_serverLock, "cm_serverLock", LOCK_HIERARCHY_SERVER_GLOBAL);
         lock_InitializeRWLock(&cm_syscfgLock, "cm_syscfgLock", LOCK_HIERARCHY_SYSCFG_GLOBAL);
@@ -886,9 +748,9 @@ void cm_SetServerNo64Bit(cm_server_t * serverp, int no64bit)
 {
     lock_ObtainMutex(&serverp->mx);
     if (no64bit)
-        serverp->flags |= CM_SERVERFLAG_NO64BIT;
+        _InterlockedOr(&serverp->flags, CM_SERVERFLAG_NO64BIT);
     else
-        serverp->flags &= ~CM_SERVERFLAG_NO64BIT;
+        _InterlockedAnd(&serverp->flags, ~CM_SERVERFLAG_NO64BIT);
     lock_ReleaseMutex(&serverp->mx);
 }
 
@@ -896,9 +758,9 @@ void cm_SetServerNoInlineBulk(cm_server_t * serverp, int no)
 {
     lock_ObtainMutex(&serverp->mx);
     if (no)
-        serverp->flags |= CM_SERVERFLAG_NOINLINEBULK;
+        _InterlockedOr(&serverp->flags, CM_SERVERFLAG_NOINLINEBULK);
     else
-        serverp->flags &= ~CM_SERVERFLAG_NOINLINEBULK;
+        _InterlockedAnd(&serverp->flags, ~CM_SERVERFLAG_NOINLINEBULK);
     lock_ReleaseMutex(&serverp->mx);
 }
 
@@ -947,11 +809,11 @@ void cm_SetServerPrefs(cm_server_t * serverp)
 	myNet    =  myAddr & netMask;
 	mySubnet =  myAddr & cm_SubnetMask[i];
 
-	if ( (serverAddr & netMask) == myNet ) 
+	if ( (serverAddr & netMask) == myNet )
 	{
 	    if ( (serverAddr & cm_SubnetMask[i]) == mySubnet)
 	    {
-		if ( serverAddr == myAddr ) 
+		if ( serverAddr == myAddr )
 		    serverp->ipRank = min(serverp->ipRank,
 					   CM_IPRANK_TOP);/* same machine */
 		else serverp->ipRank = min(serverp->ipRank,
@@ -959,7 +821,7 @@ void cm_SetServerPrefs(cm_server_t * serverp)
 	    }
 	    else serverp->ipRank = min(serverp->ipRank,CM_IPRANK_MED);
 	    /* same net */
-	}	
+	}
     } /* and of for loop */
 
     /* random between 0..15*/
@@ -972,6 +834,20 @@ cm_server_t *cm_NewServer(struct sockaddr_in *socketp, int type, cm_cell_t *cell
 
     osi_assertx(socketp->sin_family == AF_INET, "unexpected socket family");
 
+    lock_ObtainWrite(&cm_serverLock); 	/* get server lock */
+    tsp = cm_FindServer(socketp, type, TRUE);
+    if (tsp) {
+        /* we might have found a server created by set server prefs */
+        if (uuidp && !afs_uuid_is_nil(uuidp) &&
+            !(tsp->flags & CM_SERVERFLAG_UUID))
+        {
+            tsp->uuid = *uuidp;
+            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_UUID);
+        }
+        lock_ReleaseWrite(&cm_serverLock);
+        return tsp;
+    }
+
     tsp = malloc(sizeof(*tsp));
     if (tsp) {
         memset(tsp, 0, sizeof(*tsp));
@@ -979,43 +855,44 @@ cm_server_t *cm_NewServer(struct sockaddr_in *socketp, int type, cm_cell_t *cell
         tsp->cellp = cellp;
         if (uuidp && !afs_uuid_is_nil(uuidp)) {
             tsp->uuid = *uuidp;
-            tsp->flags |= CM_SERVERFLAG_UUID;
+            _InterlockedOr(&tsp->flags, CM_SERVERFLAG_UUID);
         }
         tsp->refCount = 1;
         lock_InitializeMutex(&tsp->mx, "cm_server_t mutex", LOCK_HIERARCHY_SERVER);
         tsp->addr = *socketp;
 
-        cm_SetServerPrefs(tsp); 
+        cm_SetServerPrefs(tsp);
 
-        lock_ObtainWrite(&cm_serverLock); 	/* get server lock */
         tsp->allNextp = cm_allServersp;
         cm_allServersp = tsp;
 
         switch (type) {
         case CM_SERVER_VLDB:
             cm_numVldbServers++;
-            break;      
+            break;
         case CM_SERVER_FILE:
             cm_numFileServers++;
             break;
         }
-
-        lock_ReleaseWrite(&cm_serverLock); 	/* release server lock */
-
-        if ( !(flags & CM_FLAG_NOPROBE) ) {
-            tsp->flags |= CM_SERVERFLAG_DOWN;	/* assume down; ping will mark up if available */
-            cm_PingServer(tsp);	                /* Obtain Capabilities and check up/down state */
-        }
     }
+    lock_ReleaseWrite(&cm_serverLock); 	/* release server lock */
+
+    if (!(flags & CM_FLAG_NOPROBE) && tsp) {
+        _InterlockedOr(&tsp->flags, CM_SERVERFLAG_DOWN);	/* assume down; ping will mark up if available */
+        cm_PingServer(tsp);	                                /* Obtain Capabilities and check up/down state */
+    }
+
     return tsp;
 }
 
 cm_server_t *
-cm_FindServerByIP(afs_uint32 ipaddr, unsigned short port, int type)
+cm_FindServerByIP(afs_uint32 ipaddr, unsigned short port, int type, int locked)
 {
     cm_server_t *tsp;
 
-    lock_ObtainRead(&cm_serverLock);
+    if (!locked)
+        lock_ObtainRead(&cm_serverLock);
+
     for (tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
         if (tsp->type == type &&
             tsp->addr.sin_addr.S_un.S_addr == ipaddr &&
@@ -1024,59 +901,45 @@ cm_FindServerByIP(afs_uint32 ipaddr, unsigned short port, int type)
     }
 
     /* bump ref count if we found the server */
-    if (tsp) 
+    if (tsp)
         cm_GetServerNoLock(tsp);
 
-    lock_ReleaseRead(&cm_serverLock);
+    if (!locked)
+        lock_ReleaseRead(&cm_serverLock);
 
     return tsp;
 }
 
 cm_server_t *
-cm_FindServerByUuid(afsUUID *serverUuid, int type)
+cm_FindServerByUuid(afsUUID *serverUuid, int type, int locked)
 {
     cm_server_t *tsp;
 
-    lock_ObtainRead(&cm_serverLock);
+    if (!locked)
+        lock_ObtainRead(&cm_serverLock);
+
     for (tsp = cm_allServersp; tsp; tsp = tsp->allNextp) {
         if (tsp->type == type && !afs_uuid_equal(&tsp->uuid, serverUuid))
             break;
     }
 
     /* bump ref count if we found the server */
-    if (tsp) 
+    if (tsp)
         cm_GetServerNoLock(tsp);
 
-    lock_ReleaseRead(&cm_serverLock);
+    if (!locked)
+        lock_ReleaseRead(&cm_serverLock);
 
     return tsp;
 }
 
 /* find a server based on its properties */
-cm_server_t *cm_FindServer(struct sockaddr_in *addrp, int type)
+cm_server_t *cm_FindServer(struct sockaddr_in *addrp, int type, int locked)
 {
-    cm_server_t *tsp;
-
     osi_assertx(addrp->sin_family == AF_INET, "unexpected socket value");
-        
-    lock_ObtainRead(&cm_serverLock);
-    for (tsp = cm_allServersp; tsp; tsp=tsp->allNextp) {
-        if (tsp->type == type &&
-            tsp->addr.sin_addr.s_addr == addrp->sin_addr.s_addr &&
-            (tsp->addr.sin_port == addrp->sin_port || tsp->addr.sin_port == 0))
-            break;
-    }       
 
-    /* bump ref count if we found the server */
-    if (tsp) 
-        cm_GetServerNoLock(tsp);
-
-    /* drop big table lock */
-    lock_ReleaseRead(&cm_serverLock);
-	
-    /* return what we found */
-    return tsp;
-}       
+    return cm_FindServerByIP(addrp->sin_addr.s_addr, addrp->sin_port, type, locked);
+}
 
 cm_server_vols_t *cm_NewServerVols(void) {
     cm_server_vols_t *tsvp;
@@ -1088,6 +951,10 @@ cm_server_vols_t *cm_NewServerVols(void) {
     return tsvp;
 }
 
+/*
+ * cm_NewServerRef() returns with the allocated cm_serverRef_t
+ * with a refCount of 1.
+ */
 cm_serverRef_t *cm_NewServerRef(cm_server_t *serverp, afs_uint32 volID)
 {
     cm_serverRef_t *tsrp;
@@ -1148,6 +1015,34 @@ cm_serverRef_t *cm_NewServerRef(cm_server_t *serverp, afs_uint32 volID)
     return tsrp;
 }
 
+void cm_GetServerRef(cm_serverRef_t *tsrp, int locked)
+{
+    afs_int32 refCount;
+
+    if (!locked)
+        lock_ObtainRead(&cm_serverLock);
+    refCount = InterlockedIncrement(&tsrp->refCount);
+    if (!locked)
+        lock_ReleaseRead(&cm_serverLock);
+}
+
+afs_int32 cm_PutServerRef(cm_serverRef_t *tsrp, int locked)
+{
+    afs_int32 refCount;
+
+    if (!locked)
+        lock_ObtainRead(&cm_serverLock);
+    refCount = InterlockedDecrement(&tsrp->refCount);
+    osi_assertx(refCount >= 0, "cm_serverRef_t refCount underflow");
+
+    if (!locked)
+        lock_ReleaseRead(&cm_serverLock);
+
+    return refCount;
+}
+
+
+
 LONG_PTR cm_ChecksumServerList(cm_serverRef_t *serversp)
 {
     LONG_PTR sum = 0;
@@ -1170,63 +1065,134 @@ LONG_PTR cm_ChecksumServerList(cm_serverRef_t *serversp)
 }
 
 /*
-** Insert a server into the server list keeping the list sorted in 
-** ascending order of ipRank. 
-** 
-** The refCount of the cm_serverRef_t is increased
+** Insert a server into the server list keeping the list sorted in
+** ascending order of ipRank.
+**
+** The refCount of the cm_serverRef_t is not altered.
 */
 void cm_InsertServerList(cm_serverRef_t** list, cm_serverRef_t* element)
 {
-    cm_serverRef_t	*current=*list;
-    unsigned short ipRank = element->server->ipRank;
+    cm_serverRef_t	*current;
+    unsigned short ipRank;
 
     lock_ObtainWrite(&cm_serverLock);
-    element->refCount++;                /* increase refCount */
+    /*
+     * Since we are grabbing the serverLock exclusively remove any
+     * deleted serverRef objects with a zero refcount before
+     * inserting the new item.
+     */
+    if (*list) {
+        cm_serverRef_t  **currentp = list;
+        cm_serverRef_t  **nextp = NULL;
+        cm_serverRef_t  * next = NULL;
+
+        for (currentp = list; *currentp; currentp = nextp)
+        {
+            nextp = &(*currentp)->next;
+            if ((*currentp)->refCount == 0 &&
+                (*currentp)->status == srv_deleted) {
+                next = *nextp;
+
+                if ((*currentp)->volID)
+                    cm_RemoveVolumeFromServer((*currentp)->server, (*currentp)->volID);
+                cm_FreeServer((*currentp)->server);
+                free(*currentp);
+                nextp = &next;
+            }
+        }
+    }
 
     /* insertion into empty list  or at the beginning of the list */
-    if ( !current || (current->server->ipRank > ipRank) )
+    if (!(*list))
+    {
+        element->next = NULL;
+        *list = element;
+        goto done;
+    }
+
+    /*
+     * Now that deleted entries have been removed and we know that the
+     * list was not empty, look for duplicates.  If the element we are
+     * inserting already exists, discard it.
+     */
+    for ( current = *list; current; current = current->next)
+    {
+        cm_server_t * server1 = current->server;
+        cm_server_t * server2 = element->server;
+
+        if (current->status == srv_deleted)
+            continue;
+
+        if (server1->type != server2->type)
+            continue;
+
+        if (server1->addr.sin_addr.s_addr != server2->addr.sin_addr.s_addr)
+            continue;
+
+        if ((server1->flags & CM_SERVERFLAG_UUID) != (server2->flags & CM_SERVERFLAG_UUID))
+            continue;
+
+        if ((server1->flags & CM_SERVERFLAG_UUID) &&
+            !afs_uuid_equal(&server1->uuid, &server2->uuid))
+            continue;
+
+        /* we must have a match, discard the new element */
+        free(element);
+        goto done;
+    }
+
+    ipRank = element->server->ipRank;
+
+	/* insertion at the beginning of the list */
+    if ((*list)->server->ipRank > ipRank)
     {
         element->next = *list;
         *list = element;
-        lock_ReleaseWrite(&cm_serverLock);
-        return ;	
+        goto done;
     }
-	
-    while ( current->next ) /* find appropriate place to insert */
+
+    /* find appropriate place to insert */
+    for ( current = *list; current->next; current = current->next)
     {
         if ( current->next->server->ipRank > ipRank )
             break;
-        else current = current->next;
     }
     element->next = current->next;
     current->next = element;
+
+  done:
     lock_ReleaseWrite(&cm_serverLock);
-}       
+}
 /*
 ** Re-sort the server list with the modified rank
-** returns 0 if element was changed successfully. 
+** returns 0 if element was changed successfully.
 ** returns 1 if  list remained unchanged.
 */
 long cm_ChangeRankServer(cm_serverRef_t** list, cm_server_t*	server)
 {
-    cm_serverRef_t  **current=list;
-    cm_serverRef_t	*element=0;
-
-    /* if there is max of one element in the list, nothing to sort */
-    if ( (!*current) || !((*current)->next)  )
-        return 1;		/* list unchanged: return success */
+    cm_serverRef_t  **current;
+    cm_serverRef_t   *element;
 
     lock_ObtainWrite(&cm_serverLock);
+    current=list;
+    element=0;
+
+    /* if there is max of one element in the list, nothing to sort */
+    if ( (!*current) || !((*current)->next)  ) {
+        lock_ReleaseWrite(&cm_serverLock);
+        return 1;		/* list unchanged: return success */
+    }
+
     /* if the server is on the list, delete it from list */
     while ( *current )
     {
         if ( (*current)->server == server)
         {
             element = (*current);
-            *current = (*current)->next; /* delete it */
+            *current = element->next; /* delete it */
             break;
         }
-        current = & ( (*current)->next);	
+        current = & ( (*current)->next);
     }
     lock_ReleaseWrite(&cm_serverLock);
 
@@ -1237,27 +1203,26 @@ long cm_ChangeRankServer(cm_serverRef_t** list, cm_server_t*	server)
     /* re-insert deleted element into the list with modified rank*/
     cm_InsertServerList(list, element);
 
-    /* reduce refCount which was increased by cm_InsertServerList */
-    lock_ObtainWrite(&cm_serverLock);
-    element->refCount--;
-    lock_ReleaseWrite(&cm_serverLock);
     return 0;
 }
 /*
-** If there are more than one server on the list and the first n servers on 
+** If there are more than one server on the list and the first n servers on
 ** the list have the same rank( n>1), then randomise among the first n servers.
 */
 void cm_RandomizeServer(cm_serverRef_t** list)
 {
     int 		count, picked;
-    cm_serverRef_t*	tsrp = *list, *lastTsrp;
+    cm_serverRef_t*	tsrp, *lastTsrp;
     unsigned short	lowestRank;
 
-    /* an empty list or a list with only one element */
-    if ( !tsrp || ! tsrp->next )
-        return ; 
-
     lock_ObtainWrite(&cm_serverLock);
+    tsrp = *list;
+
+    /* an empty list or a list with only one element */
+    if ( !tsrp || ! tsrp->next ) {
+        lock_ReleaseWrite(&cm_serverLock);
+        return ;
+    }
 
     /* count the number of servers with the lowest rank */
     lowestRank = tsrp->server->ipRank;
@@ -1267,19 +1232,19 @@ void cm_RandomizeServer(cm_serverRef_t** list)
             break;
         else
             count++;
-    }       	
+    }
 
     /* if there is only one server with the lowest rank, we are done */
     if ( count <= 1 ) {
         lock_ReleaseWrite(&cm_serverLock);
         return ;
-    }   
+    }
 
     picked = rand() % count;
     if ( !picked ) {
         lock_ReleaseWrite(&cm_serverLock);
         return ;
-    }   
+    }
 
     tsrp = *list;
     while (--picked >= 0)
@@ -1291,7 +1256,7 @@ void cm_RandomizeServer(cm_serverRef_t** list)
     tsrp->next     = *list; /* insert element at the beginning of list */
     *list          = tsrp;
     lock_ReleaseWrite(&cm_serverLock);
-}       
+}
 
 /* call cm_FreeServer while holding a write lock on cm_serverLock */
 void cm_FreeServer(cm_server_t* serverp)
@@ -1302,14 +1267,14 @@ void cm_FreeServer(cm_server_t* serverp)
     cm_PutServerNoLock(serverp);
     if (serverp->refCount == 0)
     {
-        /* 
+        /*
          * we need to check to ensure that all of the connections
          * for this server have a 0 refCount; otherwise, they will
-         * not be garbage collected 
+         * not be garbage collected
          *
          * must drop the cm_serverLock because cm_GCConnections
-         * obtains the cm_connLock and that comes first in the 
-         * lock hierarchy.  
+         * obtains the cm_connLock and that comes first in the
+         * lock hierarchy.
          */
         lock_ReleaseWrite(&cm_serverLock);
         cm_GCConnections(serverp);  /* connsp */
@@ -1317,9 +1282,9 @@ void cm_FreeServer(cm_server_t* serverp)
     }
 
 
-    /* 
+    /*
      * Once we have the cm_serverLock locked check to make
-     * sure the refCount is still zero before removing the 
+     * sure the refCount is still zero before removing the
      * server entirely.
      */
     if (serverp->refCount == 0) {
@@ -1327,7 +1292,7 @@ void cm_FreeServer(cm_server_t* serverp)
             switch (serverp->type) {
             case CM_SERVER_VLDB:
                 cm_numVldbServers--;
-                break;      
+                break;
             case CM_SERVER_FILE:
                 cm_numFileServers--;
                 break;
@@ -1379,19 +1344,24 @@ void cm_RemoveVolumeFromServer(cm_server_t * serverp, afs_uint32 volID)
 
 void cm_FreeServerList(cm_serverRef_t** list, afs_uint32 flags)
 {
-    cm_serverRef_t  **current = list;
-    cm_serverRef_t  **nextp = 0;
-    cm_serverRef_t  * next = 0;
-
-	if (*list == NULL)
-		return;
+    cm_serverRef_t  **current;
+    cm_serverRef_t  **nextp;
+    cm_serverRef_t  * next;
+    afs_int32         refCount;
 
     lock_ObtainWrite(&cm_serverLock);
+    current = list;
+    nextp = 0;
+    next = 0;
+
+    if (*list == NULL)
+        goto done;
 
     while (*current)
     {
         nextp = &(*current)->next;
-        if (--((*current)->refCount) == 0) {
+        refCount = cm_PutServerRef(*current, TRUE);
+        if (refCount == 0) {
             next = *nextp;
 
             if ((*current)->volID)
@@ -1408,13 +1378,15 @@ void cm_FreeServerList(cm_serverRef_t** list, afs_uint32 flags)
             current = nextp;
         }
     }
-  
+
+  done:
+
     lock_ReleaseWrite(&cm_serverLock);
 }
 
-/* dump all servers to a file. 
- * cookie is used to identify this batch for easy parsing, 
- * and it a string provided by a caller 
+/* dump all servers to a file.
+ * cookie is used to identify this batch for easy parsing,
+ * and it a string provided by a caller
  */
 int cm_DumpServers(FILE *outputFile, char *cookie, int lock)
 {
@@ -1426,12 +1398,12 @@ int cm_DumpServers(FILE *outputFile, char *cookie, int lock)
 
     if (lock)
         lock_ObtainRead(&cm_serverLock);
-  
+
     sprintf(output,
             "%s - dumping servers - cm_numFileServers=%d, cm_numVldbServers=%d\r\n",
             cookie, cm_numFileServers, cm_numVldbServers);
     WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
-  
+
     for (tsp = cm_allServersp; tsp; tsp=tsp->allNextp)
     {
         char * type;
@@ -1465,17 +1437,17 @@ int cm_DumpServers(FILE *outputFile, char *cookie, int lock)
     }
     sprintf(output, "%s - Done dumping servers.\r\n", cookie);
     WriteFile(outputFile, output, (DWORD)strlen(output), &zilch, NULL);
-  
+
     if (lock)
 	lock_ReleaseRead(&cm_serverLock);
 
-    return (0);     
+    return (0);
 }
 
-/* 
+/*
  * Determine if two servers are in fact the same.
  *
- * Returns 1 if they match, 0 if they do not 
+ * Returns 1 if they match, 0 if they do not
  */
 int cm_ServerEqual(cm_server_t *srv1, cm_server_t *srv2)
 {
@@ -1494,8 +1466,17 @@ int cm_ServerEqual(cm_server_t *srv1, cm_server_t *srv2)
         /* Both support UUID */
         if (UuidEqual((UUID *)&srv1->uuid, (UUID *)&srv2->uuid, &status))
             return 1;
-    } 
-    
+    } else {
+        if (srv1->flags & CM_SERVERFLAG_UUID)
+            return 0;
+
+        /* Neither support UUID so perform an addr/port comparison */
+        if ( srv1->addr.sin_family == srv2->addr.sin_family &&
+             srv1->addr.sin_addr.s_addr == srv2->addr.sin_addr.s_addr &&
+             srv1->addr.sin_port == srv2->addr.sin_port )
+            return 1;
+    }
+
     return 0;
 }
 
