@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -66,11 +66,9 @@ END_MESSAGE_MAP()
 /////////////////////////////////////////////////////////////////////////////
 // CMakeSymbolicLinkDlg message handlers
 
-void CMakeSymbolicLinkDlg::OnOK() 
+void CMakeSymbolicLinkDlg::OnOK()
 {
-    char space[64];
     UINT code;
-    int i;
     UpdateData(TRUE);
     CString strName;
     CString strTarget;
@@ -79,29 +77,29 @@ void CMakeSymbolicLinkDlg::OnOK()
     if (!IsPathInAfs(m_sBase)) {
         MessageBeep((UINT)-1);
         strTarget+=" - Path is not in AFS directory";
-        AfxMessageBox(strTarget,MB_OK);
+        AfxMessageBox(strTarget,MB_ICONERROR);
         return;
-    } 
+    }
     if (m_sBase.GetLength()>MAX_PATH-2)
     {
         MessageBeep((UINT)-1);
-        ShowMessageBox(IDS_CURRENT_DIRECTORY_PATH_TOO_LONG,MB_OK,IDS_CURRENT_DIRECTORY_PATH_TOO_LONG);
+        ShowMessageBox(IDS_CURRENT_DIRECTORY_PATH_TOO_LONG,MB_ICONERROR,IDS_CURRENT_DIRECTORY_PATH_TOO_LONG);
     }
     if (!SetCurrentDirectory(m_sBase))
     {
         MessageBeep((UINT)-1);
-        ShowMessageBox(IDS_UNABLE_TO_SET_CURRENT_DIRECTORY,MB_OK,IDS_UNABLE_TO_SET_CURRENT_DIRECTORY);
+        ShowMessageBox(IDS_UNABLE_TO_SET_CURRENT_DIRECTORY,MB_ICONERROR,IDS_UNABLE_TO_SET_CURRENT_DIRECTORY);
         return;
     }
     if ((code=MakeSymbolicLink(strName,strTarget))!=0){
         MessageBeep((UINT)-1);
-        ShowMessageBox(IDS_UNABLE_TO_CREATE_SYMBOLIC_LINK,MB_OK,IDS_UNABLE_TO_CREATE_SYMBOLIC_LINK,code);
+        ShowMessageBox(IDS_UNABLE_TO_CREATE_SYMBOLIC_LINK,MB_ICONERROR,IDS_UNABLE_TO_CREATE_SYMBOLIC_LINK,GetAfsError(code, strName));
         return;
     }
     CDialog::OnOK();
 }
 /*
-void CMakeSymbolicLinkDlg::OnChangeName() 
+void CMakeSymbolicLinkDlg::OnChangeName()
 {
 	CString strName;
 	m_Name.GetWindowText(strName);
@@ -113,7 +111,7 @@ void CMakeSymbolicLinkDlg::OnChangeName()
 	CheckEnableOk();
 }
 
-void CMakeSymbolicLinkDlg::OnChangeDir() 
+void CMakeSymbolicLinkDlg::OnChangeDir()
 {
 	m_Dir.GetWindowText(m_strDir);
     if (!IsPathInAfs(m_strDir)) {
@@ -126,17 +124,17 @@ void CMakeSymbolicLinkDlg::OnChangeDir()
 void CMakeSymbolicLinkDlg::CheckEnableOk()
 {
 	BOOL bEnable = FALSE;
-	
+
 	if ((m_strName.GetLength() > 0) && (m_strDir.GetLength() > 0))
 		bEnable = TRUE;
 
 	m_OK.EnableWindow(bEnable);
 }
 
-BOOL CMakeSymbolicLinkDlg::OnInitDialog() 
+BOOL CMakeSymbolicLinkDlg::OnInitDialog()
 {
 	CDialog::OnInitDialog();
-	
+
 //	m_Dir.SetWindowText(m_strDir);
 //	m_Name.SetWindowText(m_strName);
 	UpdateData(FALSE);
