@@ -32,20 +32,21 @@ extern afs_rwlock_t afs_xcbhash;
 int
 #if	defined(AFS_SUN5_ENV) || defined(AFS_SGI_ENV)
 afs_link(OSI_VC_DECL(adp), struct vcache *avc, char *aname, 
-	 AFS_UCRED *acred)
+	 afs_ucred_t *acred)
 #else
 afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname, 
-	 AFS_UCRED *acred)
+	 afs_ucred_t *acred)
 #endif
 {
     struct vrequest treq;
-    register struct dcache *tdc;
-    register afs_int32 code;
-    register struct afs_conn *tc;
+    struct dcache *tdc;
+    afs_int32 code;
+    struct afs_conn *tc;
     afs_size_t offset, len;
     struct AFSFetchStatus OutFidStatus, OutDirStatus;
     struct AFSVolSync tsync;
     struct afs_fakestat_state vfakestate, dfakestate;
+    struct rx_connection *rxconn;
     XSTATS_DECLS;
     OSI_VC_CONVERT(adp);
 
@@ -97,12 +98,12 @@ afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname,
     tdc = afs_GetDCache(adp, (afs_size_t) 0, &treq, &offset, &len, 1);	/* test for error below */
     ObtainWriteLock(&adp->lock, 145);
     do {
-	tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK);
+	tc = afs_Conn(&adp->f.fid, &treq, SHARED_LOCK, &rxconn);
 	if (tc) {
 	    XSTATS_START_TIME(AFS_STATS_FS_RPCIDX_LINK);
 	    RX_AFS_GUNLOCK();
 	    code =
-		RXAFS_Link(tc->id, (struct AFSFid *)&adp->f.fid.Fid, aname,
+		RXAFS_Link(rxconn, (struct AFSFid *)&adp->f.fid.Fid, aname,
 			   (struct AFSFid *)&avc->f.fid.Fid, &OutFidStatus,
 			   &OutDirStatus, &tsync);
 	    RX_AFS_GLOCK();
@@ -111,7 +112,7 @@ afs_link(struct vcache *avc, OSI_VC_DECL(adp), char *aname,
 	} else
 	    code = -1;
     } while (afs_Analyze
-	     (tc, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_LINK,
+	     (tc, rxconn, code, &adp->f.fid, &treq, AFS_STATS_FS_RPCIDX_LINK,
 	      SHARED_LOCK, NULL));
 
     if (code) {
