@@ -238,7 +238,7 @@ afs_MarkServerUpOrDown(struct srvAddr *sa, int a_isDown)
 
 
 afs_int32
-afs_ServerDown(struct srvAddr *sa)
+afs_ServerDown(struct srvAddr *sa, int code)
 {
     struct server *aserver = sa->server;
 
@@ -248,10 +248,10 @@ afs_ServerDown(struct srvAddr *sa)
     afs_MarkServerUpOrDown(sa, SRVR_ISDOWN);
     if (sa->sa_portal == aserver->cell->vlport)
 	print_internet_address
-	    ("afs: Lost contact with volume location server ", sa, "", 1);
+	    ("afs: Lost contact with volume location server ", sa, "", 1, code);
     else
 	print_internet_address("afs: Lost contact with file server ", sa, "",
-			       1);
+			       1, code);
     return 1;
 }				/*ServerDown */
 
@@ -299,7 +299,8 @@ CheckVLServer(struct srvAddr *sa, struct vrequest *areq)
 	return;			/* can't do much */
 
     tc = afs_ConnByHost(aserver, aserver->cell->vlport,
-			aserver->cell->cellNum, areq, 1, SHARED_LOCK, &rxconn);
+			aserver->cell->cellNum, areq, 1, SHARED_LOCK, 0,
+			&rxconn);
     if (!tc)
 	return;
     rx_SetConnDeadTime(rxconn, 3);
@@ -317,7 +318,7 @@ CheckVLServer(struct srvAddr *sa, struct vrequest *areq)
 	if (tc->srvr == sa) {
 	    afs_MarkServerUpOrDown(sa, 0);
 	    print_internet_address("afs: volume location server ", sa,
-				   " is back up", 2);
+				   " is back up", 2, code);
 	}
     }
 
@@ -506,6 +507,7 @@ ForceAllNewConnections(void)
     }
 
     addrs = afs_osi_Alloc(srvAddrCount * sizeof(*addrs));
+    osi_Assert(addrs != NULL);
     j = 0;
     for (i = 0; i < NSERVERS; i++) {
 	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
@@ -523,218 +525,103 @@ ForceAllNewConnections(void)
     }
 }
 
-/* check down servers (if adown), or running servers (if !adown) */
-void
-afs_CheckServers(int adown, struct cell *acellp)
+static void
+CkSrv_MarkUpDown(struct afs_conn **conns, int nconns, afs_int32 *results)
 {
-    struct vrequest treq;
-    struct server *ts;
     struct srvAddr *sa;
     struct afs_conn *tc;
-    afs_int32 i, j;
-    afs_int32 code;
+    afs_int32 i;
+
+    for(i = 0; i < nconns; i++){
+	tc = conns[i];
+	sa = tc->srvr;
+
+	if (( results[i] >= 0 ) && (sa->sa_flags & SRVADDR_ISDOWN) &&
+	    (tc->srvr == sa)) {
+	    /* server back up */
+	    print_internet_address("afs: file server ", sa, " is back up", 2,
+			           results[i]);
+
+	    ObtainWriteLock(&afs_xserver, 244);
+	    ObtainWriteLock(&afs_xsrvAddr, 245);
+	    afs_MarkServerUpOrDown(sa, 0);
+	    ReleaseWriteLock(&afs_xsrvAddr);
+	    ReleaseWriteLock(&afs_xserver);
+
+	    if (afs_waitForeverCount) {
+		afs_osi_Wakeup(&afs_waitForever);
+	    }
+	} else {
+	    if (results[i] < 0) {
+		/* server crashed */
+		afs_ServerDown(sa, results[i]);
+		ForceNewConnections(sa);  /* multi homed clients */
+	    }
+	}
+    }
+}
+
+void
+CkSrv_SetTime(struct rx_connection **rxconns, int nconns, int nservers,
+	      struct afs_conn **conns, struct srvAddr **addrs)
+{
+    struct afs_conn *tc;
     afs_int32 start, end = 0, delta;
     osi_timeval_t tv;
-    struct unixuser *tu;
+    struct srvAddr *sa;
+    afs_int32 *conntimer, *results, *deltas;
+    afs_int32 i = 0;
     char tbuffer[CVBS];
-    int srvAddrCount;
-    struct srvAddr **addrs;
-    struct afs_conn **conns;
-    int nconns;
-    struct rx_connection **rxconns;
-    afs_int32 *conntimer, *deltas, *results;
-    Capabilities *caps = NULL;
 
-    AFS_STATCNT(afs_CheckServers);
+    conntimer = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    osi_Assert(conntimer != NULL);
+    results = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    osi_Assert(results != NULL);
+    deltas = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    osi_Assert(deltas != NULL);
 
-    /*
-     * No sense in doing the server checks if we are running in disconnected
-     * mode
-     */
-    if (AFS_IS_DISCONNECTED)
-        return;
-
-    conns = (struct afs_conn **)0;
-    rxconns = (struct rx_connection **) 0;
-    conntimer = 0;
-    nconns = 0;
-
-    if ((code = afs_InitReq(&treq, afs_osi_credp)))
-	return;
-    ObtainReadLock(&afs_xserver);	/* Necessary? */
-    ObtainReadLock(&afs_xsrvAddr);
-
-    srvAddrCount = 0;
-    for (i = 0; i < NSERVERS; i++) {
-	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
-	    srvAddrCount++;
-	}
-    }
-
-    addrs = afs_osi_Alloc(srvAddrCount * sizeof(*addrs));
-    j = 0;
-    for (i = 0; i < NSERVERS; i++) {
-	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
-	    if (j >= srvAddrCount)
-		break;
-	    addrs[j++] = sa;
-	}
-    }
-
-    ReleaseReadLock(&afs_xsrvAddr);
-    ReleaseReadLock(&afs_xserver);
-
-    conns = (struct afs_conn **)afs_osi_Alloc(j * sizeof(struct afs_conn *));
-    rxconns = (struct rx_connection **)afs_osi_Alloc(j * sizeof(struct rx_connection *));
-    conntimer = (afs_int32 *)afs_osi_Alloc(j * sizeof (afs_int32));
-    deltas = (afs_int32 *)afs_osi_Alloc(j * sizeof (afs_int32));
-    results = (afs_int32 *)afs_osi_Alloc(j * sizeof (afs_int32));
-
-    caps = (Capabilities *)afs_osi_Alloc(j * sizeof (Capabilities));
-    memset(caps, 0, j * sizeof(Capabilities));
-
-    for (i = 0; i < j; i++) {
-	struct rx_connection *rxconn;
-
-	deltas[i] = 0;
-	sa = addrs[i];
-	ts = sa->server;
-	if (!ts)
-	    continue;
-
-	/* See if a cell to check was specified.  If it is spec'd and not
-	 * this server's cell, just skip the server.
-	 */
-	if (acellp && acellp != ts->cell)
-	    continue;
-
-	if ((!adown && (sa->sa_flags & SRVADDR_ISDOWN))
-	    || (adown && !(sa->sa_flags & SRVADDR_ISDOWN)))
-	    continue;
-
-	/* check vlserver with special code */
-	if (sa->sa_portal == AFS_VLPORT) {
-	    CheckVLServer(sa, &treq);
-	    continue;
-	}
-
-	if (!ts->cell)		/* not really an active server, anyway, it must */
-	    continue;		/* have just been added by setsprefs */
-
-	/* get a connection, even if host is down; bumps conn ref count */
-	tu = afs_GetUser(treq.uid, ts->cell->cellNum, SHARED_LOCK);
-	tc = afs_ConnBySA(sa, ts->cell->fsport, ts->cell->cellNum, tu,
-			  1 /*force */ , 1 /*create */ , SHARED_LOCK, &rxconn);
-	afs_PutUser(tu, SHARED_LOCK);
-	if (!tc)
-	    continue;
-
-	if ((sa->sa_flags & SRVADDR_ISDOWN) || afs_HaveCallBacksFrom(sa->server)
-	    || (tc->srvr->server == afs_setTimeHost)) {
-	    conns[nconns]=tc;
-	    rxconns[nconns]=rxconn;
-	    if (sa->sa_flags & SRVADDR_ISDOWN) {
-		rx_SetConnDeadTime(rxconn, 3);
-		conntimer[nconns]=1;
-	    } else {
-		conntimer[nconns]=0;
-	    }
-	    nconns++;
-	}
-    } /* Outer loop over addrs */
+    /* make sure we're starting from zero */
+    memset(&deltas, 0, sizeof(deltas));
 
+    start = osi_Time();         /* time the gettimeofday call */
     AFS_GUNLOCK();
-    multi_Rx(rxconns,nconns)
-      {
-	multi_RXAFS_GetCapabilities(&caps[multi_i]);
-	results[multi_i] = multi_error;
-      } multi_End;
+    if ( afs_setTimeHost == NULL ) {
+	multi_Rx(rxconns,nconns)
+	{
+	    tv.tv_sec = tv.tv_usec = 0;
+	    multi_RXAFS_GetTime(
+		(afs_uint32 *)&tv.tv_sec, (afs_uint32 *)&tv.tv_usec);
+	    tc = conns[multi_i];
+	    sa = tc->srvr;
+	    if (conntimer[multi_i] == 1)
+		rx_SetConnDeadTime(tc->id, afs_rx_deadtime);
+	    end = osi_Time();
+	    results[multi_i]=multi_error;
+	    if ((start == end) && !multi_error)
+		deltas[multi_i] = end - tv.tv_sec;
+	} multi_End;
+    } else {			/* find and query setTimeHost only */
+	for ( i = 0 ; i < nservers ; i++ ) {
+	    if ( conns[i] == NULL || conns[i]->srvr == NULL )
+		continue;
+	    if ( conns[i]->srvr->server == afs_setTimeHost ) {
+		tv.tv_sec = tv.tv_usec = 0;
+		results[i] = RXAFS_GetTime(rxconns[i],
+					   (afs_uint32 *)&tv.tv_sec,
+					   (afs_uint32 *)&tv.tv_usec);
+		end = osi_Time();
+		if ((start == end) && !results[i])
+		    deltas[i] = end - tv.tv_sec;
+		break;
+	    }
+	}
+    }
     AFS_GLOCK();
 
-    for ( i = 0 ; i < nconns ; i++ ) {
-	ts = addrs[i]->server;
-	if ( !ts )
-	    continue;
-	ts->capabilities = 0;
-	ts->flags |= SCAPS_KNOWN;
-	if ( results[i] == RXGEN_OPCODE ) {
-	    /* Mark server as up - it responded */
-	    results[i] = 0;
-	    continue;
-	}
-	if ( results[i] >= 0 )
-	    /* we currently handle 32-bits of capabilities */
-	    if (caps[i].Capabilities_len > 0) {
-		ts->capabilities = caps[i].Capabilities_val[0];
-		xdr_free((xdrproc_t)xdr_Capabilities, &caps[i]);
-		caps[i].Capabilities_val = NULL;
-		caps[i].Capabilities_len = 0;
-	    }
-    }
-
-    if ( afs_setTime != 0 ) {
-	start = osi_Time();         /* time the gettimeofday call */
-	AFS_GUNLOCK();
-	if ( afs_setTimeHost == NULL ) {
-	    multi_Rx(rxconns,nconns)
-	      {
-		tv.tv_sec = tv.tv_usec = 0;
-		multi_RXAFS_GetTime(
-			(afs_uint32 *)&tv.tv_sec, (afs_uint32 *)&tv.tv_usec);
-		tc = conns[multi_i];
-		sa = tc->srvr;
-		if (conntimer[multi_i] == 1)
-		    rx_SetConnDeadTime(rxconns[multi_i], afs_rx_deadtime);
-		end = osi_Time();
-		results[multi_i]=multi_error;
-		if ((start == end) && !multi_error)
-		  deltas[multi_i] = end - tv.tv_sec;
-
-	      } multi_End;
-	    }
-	else {			/* find and query setTimeHost only */
-	    for ( i = 0 ; i < j ; i++ ) {
-		if ( conns[i] == NULL || conns[i]->srvr == NULL )
-		    continue;
-		if ( conns[i]->srvr->server == afs_setTimeHost ) {
-		    tv.tv_sec = tv.tv_usec = 0;
-		    results[i] = RXAFS_GetTime(rxconns[i],
-				(afs_uint32 *)&tv.tv_sec, (afs_uint32 *)&tv.tv_usec);
-		    end = osi_Time();
-		    if ((start == end) && !results[i])
-			deltas[i] = end - tv.tv_sec;
-		    break;
-		}
-	    }
-	}
-	AFS_GLOCK();
-    }
-
-    for(i=0;i<nconns;i++){
-      tc = conns[i];
-      sa = tc->srvr;
-
-      if (( results[i] >= 0 ) && (sa->sa_flags & SRVADDR_ISDOWN) && (tc->srvr == sa)) {
-	/* server back up */
-	print_internet_address("afs: file server ", sa, " is back up", 2);
-
-	ObtainWriteLock(&afs_xserver, 244);
-	ObtainWriteLock(&afs_xsrvAddr, 245);
-	afs_MarkServerUpOrDown(sa, 0);
-	ReleaseWriteLock(&afs_xsrvAddr);
-	ReleaseWriteLock(&afs_xserver);
-
-	if (afs_waitForeverCount) {
-	  afs_osi_Wakeup(&afs_waitForever);
-	}
-      } else {
-	  if ((results[i] < 0) && (results[i] != RXGEN_OPCODE)) {
-	      /* server crashed */
-	      afs_ServerDown(sa);
-	      ForceNewConnections(sa);  /* multi homed clients */
-	  }
-      }
-    }
+    if ( afs_setTimeHost == NULL )
+	CkSrv_MarkUpDown(conns, nconns, results);
+    else /* We lack info for other than this host */
+	CkSrv_MarkUpDown(&conns[i], 1, &results[i]);
 
     /*
      * If we're supposed to set the time, and the call worked
@@ -781,19 +668,19 @@ afs_CheckServers(int adown, struct cell *acellp)
 			    afs_strcat(msgbuf, ", via ");
 			    print_internet_address(msgbuf, sa,
 						   "); clock is still fast.",
-						   0);
+						   0, 0);
 			} else {
 			    afs_strcat(msgbuf,
 				       afs_cv2string(&tbuffer[CVBS], delta));
 			    afs_strcat(msgbuf, " seconds (via ");
-			    print_internet_address(msgbuf, sa, ").", 0);
+			    print_internet_address(msgbuf, sa, ").", 0, 0);
 			}
 		    } else {
 			strcpy(msgbuf, "afs: setting clock ahead ");
 			afs_strcat(msgbuf,
 				   afs_cv2string(&tbuffer[CVBS], -delta));
 			afs_strcat(msgbuf, " seconds (via ");
-			print_internet_address(msgbuf, sa, ").", 0);
+			print_internet_address(msgbuf, sa, ").", 0, 0);
 		    }
                     /* We're only going to set it once; why bother looping? */
 		    break;
@@ -801,7 +688,205 @@ afs_CheckServers(int adown, struct cell *acellp)
 	    }
 	}
     }
+    afs_osi_Free(conntimer, nservers * sizeof(afs_int32));
+    afs_osi_Free(deltas, nservers * sizeof(afs_int32));
+    afs_osi_Free(results, nservers * sizeof(afs_int32));
+}
+
+void
+CkSrv_GetCaps(struct rx_connection **rxconns, int nconns, int nservers,
+	      struct afs_conn **conns, struct srvAddr **addrs)
+{
+    Capabilities *caps;
+    afs_int32 *results;
+    afs_int32 i;
+    struct server *ts;
+
+    caps = afs_osi_Alloc(nservers * sizeof (Capabilities));
+    osi_Assert(caps != NULL);
+    memset(caps, 0, nservers * sizeof(Capabilities));
+
+    results = afs_osi_Alloc(nservers * sizeof (afs_int32));
+    osi_Assert(results != NULL);
+
+    AFS_GUNLOCK();
+    multi_Rx(rxconns,nconns)
+      {
+	multi_RXAFS_GetCapabilities(&caps[multi_i]);
+	results[multi_i] = multi_error;
+      } multi_End;
+    AFS_GLOCK();
+
+    for ( i = 0 ; i < nconns ; i++ ) {
+	ts = addrs[i]->server;
+	if ( !ts )
+	    continue;
+	ts->capabilities = 0;
+	ts->flags |= SCAPS_KNOWN;
+	if ( results[i] == RXGEN_OPCODE ) {
+	    /* Mark server as up - it responded */
+	    results[i] = 0;
+	    continue;
+	}
+	if ( results[i] >= 0 )
+	    /* we currently handle 32-bits of capabilities */
+	    if (caps[i].Capabilities_len > 0) {
+		ts->capabilities = caps[i].Capabilities_val[0];
+		xdr_free((xdrproc_t)xdr_Capabilities, &caps[i]);
+		caps[i].Capabilities_val = NULL;
+		caps[i].Capabilities_len = 0;
+	    }
+    }
+    CkSrv_MarkUpDown(conns, nconns, results);
+
+    afs_osi_Free(caps, nservers * sizeof(Capabilities));
+    afs_osi_Free(results, nservers * sizeof(afs_int32));
+}
+
+/* check down servers (if adown), or running servers (if !adown) */
+void
+afs_CheckServers(int adown, struct cell *acellp)
+{
+    afs_LoopServers(adown?AFS_LS_DOWN:AFS_LS_UP, acellp, 1, CkSrv_GetCaps,
+		    afs_setTime?CkSrv_SetTime:NULL);
+}
+
+/* adown: AFS_LS_UP   - check only up
+ *        AFS_LS_DOWN - check only down.
+ *        AFS_LS_ALL  - check all */
+void
+afs_LoopServers(int adown, struct cell *acellp, int vlalso,
+		void (*func1) (struct rx_connection **rxconns, int nconns,
+			       int nservers, struct afs_conn **conns,
+			       struct srvAddr **addrs),
+		void (*func2) (struct rx_connection **rxconns, int nconns,
+			       int nservers, struct afs_conn **conns,
+			       struct srvAddr **addrs))
+{
+    struct vrequest treq;
+    struct server *ts;
+    struct srvAddr *sa;
+    struct afs_conn *tc = NULL;
+    afs_int32 i, j;
+    afs_int32 code;
+    struct unixuser *tu;
+    int srvAddrCount;
+    struct srvAddr **addrs;
+    struct afs_conn **conns;
+    int nconns;
+    struct rx_connection **rxconns;
+    afs_int32 *conntimer, *results;
+
+    AFS_STATCNT(afs_CheckServers);
+
+    /*
+     * No sense in doing the server checks if we are running in disconnected
+     * mode
+     */
+    if (AFS_IS_DISCONNECTED)
+        return;
+
+    conns = (struct afs_conn **)0;
+    rxconns = (struct rx_connection **) 0;
+    conntimer = 0;
+    nconns = 0;
+
+    if ((code = afs_InitReq(&treq, afs_osi_credp)))
+	return;
+    ObtainReadLock(&afs_xserver);	/* Necessary? */
+    ObtainReadLock(&afs_xsrvAddr);
+
+    srvAddrCount = 0;
+    for (i = 0; i < NSERVERS; i++) {
+	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
+	    srvAddrCount++;
+	}
+    }
+
+    addrs = afs_osi_Alloc(srvAddrCount * sizeof(*addrs));
+    osi_Assert(addrs != NULL);
+    j = 0;
+    for (i = 0; i < NSERVERS; i++) {
+	for (sa = afs_srvAddrs[i]; sa; sa = sa->next_bkt) {
+	    if (j >= srvAddrCount)
+		break;
+	    addrs[j++] = sa;
+	}
+    }
+
+    ReleaseReadLock(&afs_xsrvAddr);
+    ReleaseReadLock(&afs_xserver);
+
+    conns = afs_osi_Alloc(j * sizeof(struct afs_conn *));
+    osi_Assert(conns != NULL);
+    rxconns = afs_osi_Alloc(j * sizeof(struct rx_connection *));
+    osi_Assert(rxconns != NULL);
+    conntimer = afs_osi_Alloc(j * sizeof (afs_int32));
+    osi_Assert(conntimer != NULL);
+    results = afs_osi_Alloc(j * sizeof (afs_int32));
+    osi_Assert(results != NULL);
+
+    for (i = 0; i < j; i++) {
+	struct rx_connection *rxconn;
+
+	sa = addrs[i];
+	ts = sa->server;
+	if (!ts)
+	    continue;
+
+	/* See if a cell to check was specified.  If it is spec'd and not
+	 * this server's cell, just skip the server.
+	 */
+	if (acellp && acellp != ts->cell)
+	    continue;
+
+	if (((adown==AFS_LS_DOWN) && !(sa->sa_flags & SRVADDR_ISDOWN))
+	    || ((adown==AFS_LS_UP) && (sa->sa_flags & SRVADDR_ISDOWN)))
+	    continue;
+
+	/* check vlserver with special code */
+	if (sa->sa_portal == AFS_VLPORT) {
+	    if (vlalso)
+		CheckVLServer(sa, &treq);
+	    continue;
+	}
+
+	if (!ts->cell)		/* not really an active server, anyway, it must */
+	    continue;		/* have just been added by setsprefs */
+
+	/* get a connection, even if host is down; bumps conn ref count */
+	tu = afs_GetUser(treq.uid, ts->cell->cellNum, SHARED_LOCK);
+	tc = afs_ConnBySA(sa, ts->cell->fsport, ts->cell->cellNum, tu,
+			  1 /*force */ , 1 /*create */ , SHARED_LOCK, 0,
+			  &rxconn);
+	afs_PutUser(tu, SHARED_LOCK);
+	if (!tc)
+	    continue;
+
+	if ((sa->sa_flags & SRVADDR_ISDOWN) || afs_HaveCallBacksFrom(sa->server)
+	    || (tc->srvr->server == afs_setTimeHost)) {
+	    conns[nconns]=tc;
+	    rxconns[nconns]=rxconn;
+	    if (sa->sa_flags & SRVADDR_ISDOWN) {
+		rx_SetConnDeadTime(rxconn, 3);
+		conntimer[nconns]=1;
+	    } else {
+		conntimer[nconns]=0;
+	    }
+	    nconns++;
+	} else /* not holding, kill ref */
+	    afs_PutConn(tc, rxconn, SHARED_LOCK);
+    } /* Outer loop over addrs */
+
+    (*func1)(rxconns, nconns, j, conns, addrs);
+
+    if (func2) {
+	(*func2)(rxconns, nconns, j, conns, addrs);
+    }
+
     for (i = 0; i < nconns; i++) {
+	if (conntimer[i] == 1)
+	    rx_SetConnDeadTime(tc->id, afs_rx_deadtime);
 	afs_PutConn(conns[i], rxconns[i], SHARED_LOCK);     /* done with it now */
     }
 
@@ -809,9 +894,7 @@ afs_CheckServers(int adown, struct cell *acellp)
     afs_osi_Free(conns, j * sizeof(struct afs_conn *));
     afs_osi_Free(rxconns, j * sizeof(struct rx_connection *));
     afs_osi_Free(conntimer, j * sizeof(afs_int32));
-    afs_osi_Free(deltas, j * sizeof(afs_int32));
     afs_osi_Free(results, j * sizeof(afs_int32));
-    afs_osi_Free(caps, j * sizeof(Capabilities));
 
 } /*afs_CheckServers*/
 
@@ -1194,7 +1277,7 @@ afsi_SetServerIPRank(struct srvAddr *sa, afs_int32 addr,
     return;
 }
 #else /* AFS_USERSPACE_IP_ADDR */
-#if (! defined(AFS_SUN5_ENV)) && (! defined(AFS_DARWIN_ENV)) && (! defined(AFS_OBSD47_ENV)) && defined(USEIFADDR)
+#if (! defined(AFS_SUN5_ENV)) && (! defined(AFS_DARWIN_ENV)) && (! defined(AFS_OBSD47_ENV)) && (! defined(AFS_FBSD_ENV)) && defined(USEIFADDR)
 void
 afsi_SetServerIPRank(struct srvAddr *sa, struct in_ifaddr *ifa)
 {
@@ -1231,7 +1314,7 @@ afsi_SetServerIPRank(struct srvAddr *sa, struct in_ifaddr *ifa)
 #endif /* IFF_POINTTOPOINT */
 }
 #endif /*(!defined(AFS_SUN5_ENV)) && defined(USEIFADDR) */
-#if (defined(AFS_DARWIN_ENV) || defined(AFS_OBSD47_ENV)) && defined(USEIFADDR)
+#if (defined(AFS_DARWIN_ENV) || defined(AFS_OBSD47_ENV) || defined(AFS_FBSD_ENV)) && defined(USEIFADDR)
 #ifndef afs_min
 #define afs_min(A,B) ((A)<(B)) ? (A) : (B)
 #endif
@@ -1240,7 +1323,11 @@ afsi_SetServerIPRank(struct srvAddr *sa, rx_ifaddr_t ifa)
 {
     struct sockaddr sout;
     struct sockaddr_in *sin;
+#if defined(AFS_DARWIN80_ENV) && !defined(UKERNEL)
     int t;
+#else
+    void *t;
+#endif
 
     afs_uint32 subnetmask, myAddr, myNet, myDstaddr, mySubnet, netMask;
     afs_uint32 serverAddr;
@@ -1564,7 +1651,7 @@ afs_SetServerPrefs(struct srvAddr *sa)
 #else
 	  TAILQ_FOREACH(ifa, &in_ifaddrhead, ia_link) {
 #endif
-	    afsi_SetServerIPRank(sa, ifa);
+	    afsi_SetServerIPRank(sa, &ifa->ia_ifa);
     }}
 #elif defined(AFS_OBSD_ENV)
     {
@@ -1608,30 +1695,28 @@ afs_SetServerPrefs(struct srvAddr *sa)
 /* afs_FlushServer()
  * The addresses on this server struct has changed in some way and will
  * clean up all other structures that may reference it.
- * The afs_xserver and afs_xsrvAddr locks are assumed taken.
+ * The afs_xserver, afs_xvcb and afs_xsrvAddr locks are assumed taken.
  */
-void
-afs_FlushServer(struct server *srvp)
+static void
+afs_FlushServer(struct server *srvp, struct volume *tv)
 {
     afs_int32 i;
     struct server *ts, **pts;
 
     /* Find any volumes residing on this server and flush their state */
-      afs_ResetVolumes(srvp);
+    afs_ResetVolumes(srvp, tv);
 
     /* Flush all callbacks in the all vcaches for this specific server */
-      afs_FlushServerCBs(srvp);
+    afs_FlushServerCBs(srvp);
 
     /* Remove all the callbacks structs */
     if (srvp->cbrs) {
 	struct afs_cbr *cb, *cbnext;
 
-	  ObtainWriteLock(&afs_xvcb, 300);
 	for (cb = srvp->cbrs; cb; cb = cbnext) {
 	    cbnext = cb->next;
 	    afs_FreeCBR(cb);
 	} srvp->cbrs = (struct afs_cbr *)0;
-	ReleaseWriteLock(&afs_xvcb);
     }
 
     /* If no more srvAddr structs hanging off of this server struct,
@@ -1666,9 +1751,10 @@ afs_FlushServer(struct server *srvp)
  * remains connected to a server struct.
  * The afs_xserver and afs_xsrvAddr locks are assumed taken.
  *    It is not removed from the afs_srvAddrs hash chain.
+ * If resetting volumes, do not reset volume tv
  */
-void
-afs_RemoveSrvAddr(struct srvAddr *sap)
+static void
+afs_RemoveSrvAddr(struct srvAddr *sap, struct volume *tv)
 {
     struct srvAddr **psa, *sa;
     struct server *srv;
@@ -1687,7 +1773,7 @@ afs_RemoveSrvAddr(struct srvAddr *sap)
 	sa->server = 0;
 
 	/* Flush the server struct since it's IP address has changed */
-	afs_FlushServer(srv);
+	afs_FlushServer(srv, tv);
     }
 }
 
@@ -1715,8 +1801,7 @@ afs_GetCapabilities(struct server *ts)
     if ( !tu )
 	return;
     tc = afs_ConnBySA(ts->addr, ts->cell->fsport, ts->cell->cellNum, tu, 0, 1,
-		      SHARED_LOCK,
-		      &rxconn);
+		      SHARED_LOCK, 0, &rxconn);
     afs_PutUser(tu, SHARED_LOCK);
     if ( !tc )
 	return;
@@ -1726,11 +1811,10 @@ afs_GetCapabilities(struct server *ts)
     ObtainWriteLock(&afs_xserver, 723);
     /* we forced a conn above; important we mark it down if needed */
     if ((code < 0) && (code != RXGEN_OPCODE)) {
-	afs_PutConn(tc, rxconn, SHARED_LOCK);
-	afs_ServerDown(tc->srvr);
+	afs_ServerDown(tc->srvr, code);
 	ForceNewConnections(tc->srvr); /* multi homed clients */
     }
-
+    afs_PutConn(tc, rxconn, SHARED_LOCK);
     if ( code && code != RXGEN_OPCODE ) {
 	afs_warn("RXAFS_GetCapabilities failed with code %d\n", code);
 	/* better not be anything to free. we failed! */
@@ -1748,20 +1832,56 @@ afs_GetCapabilities(struct server *ts)
 
 }
 
-/* afs_GetServer()
- * Return an updated and properly initialized server structure
- * corresponding to the server ID, cell, and port specified.
- * If one does not exist, then one will be created.
- * aserver and aport must be in NET byte order.
+static struct server *
+afs_SearchServer(u_short aport, afsUUID * uuidp, afs_int32 locktype,
+		 struct server **oldts, afs_int32 addr_uniquifier)
+{
+    struct server *ts = afs_FindServer(0, aport, uuidp, locktype);
+    if (ts && (ts->sr_addr_uniquifier == addr_uniquifier) && ts->addr) {
+	/* Found a server struct that is multihomed and same
+	 * uniqufier (same IP addrs). The above if statement is the
+	 * same as in InstallUVolumeEntry().
+	 */
+	return ts;
+    }
+    if (ts)
+	*oldts = ts;		/* Will reuse if same uuid */
+    return NULL;
+}
+
+/*!
+ * Return an updated and properly initialized server structure.
+ *
+ * Takes a server ID, cell, and port.
+ * If server does not exist, then one will be created.
+ * @param[in] aserverp
+ *      The server address in network byte order
+ * @param[in] nservers
+ *      The number of IP addresses claimed by the server
+ * @param[in] acell
+ *      The cell the server is in
+ * @param[in] aport
+ *      The port for the server (fileserver or vlserver) in network byte order
+ * @param[in] locktype
+ *      The type of lock to hold when iterating server hash (unused).
+ * @param[in] uuidp
+ *      The uuid for servers supporting one.
+ * @param[in] addr_uniquifier
+ *      The vldb-provider per-instantiated-server uniquifer counter.
+ * @param[in] tv
+ *      A volume not to reset information for if the server addresses
+ *      changed.
+ *
+ * @return
+ *      A server structure matching the request.
  */
 struct server *
-afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
+afs_GetServer(afs_uint32 *aserverp, afs_int32 nservers, afs_int32 acell,
 	      u_short aport, afs_int32 locktype, afsUUID * uuidp,
-	      afs_int32 addr_uniquifier)
+	      afs_int32 addr_uniquifier, struct volume *tv)
 {
     struct server *oldts = 0, *ts, *newts, *orphts = 0;
     struct srvAddr *oldsa, *newsa, *nextsa, *orphsa;
-    u_short fsport;
     afs_int32 iphash, k, srvcount = 0;
     unsigned int srvhash;
 
@@ -1772,7 +1892,7 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
     /* Check if the server struct exists and is up to date */
     if (!uuidp) {
 	if (nservers != 1)
-	    panic("afs_GetServer: incorect count of servers");
+	    panic("afs_GetServer: incorrect count of servers");
 	ObtainReadLock(&afs_xsrvAddr);
 	ts = afs_FindServer(aserverp[0], aport, NULL, locktype);
 	ReleaseReadLock(&afs_xsrvAddr);
@@ -1786,29 +1906,44 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
     } else {
 	if (nservers <= 0)
 	    panic("afs_GetServer: incorrect count of servers");
-	ts = afs_FindServer(0, aport, uuidp, locktype);
-	if (ts && (ts->sr_addr_uniquifier == addr_uniquifier) && ts->addr) {
-	    /* Found a server struct that is multihomed and same
-	     * uniqufier (same IP addrs). The above if statement is the
-	     * same as in InstallUVolumeEntry().
-	     */
+
+	ts = afs_SearchServer(aport, uuidp, locktype, &oldts, addr_uniquifier);
+	if (ts) {
 	    ReleaseSharedLock(&afs_xserver);
 	    return ts;
 	}
-	if (ts)
-	    oldts = ts;		/* Will reuse if same uuid */
     }
 
-    UpgradeSToWLock(&afs_xserver, 36);
-    ObtainWriteLock(&afs_xsrvAddr, 116);
+    /*
+     * Lock hierarchy requires xvcb, then xserver. We *have* xserver.
+     * Do a little dance and see if we can grab xvcb. If not, we
+     * need to recheck that oldts is still right after a drop and reobtain.
+     */
+    if (EWOULDBLOCK == NBObtainWriteLock(&afs_xvcb, 300)) {
+	ReleaseSharedLock(&afs_xserver);
+	ObtainWriteLock(&afs_xvcb, 299);
+	ObtainWriteLock(&afs_xserver, 35);
 
+	/* we don't know what changed while we didn't hold the lock */
+	oldts = 0;
+	ts = afs_SearchServer(aport, uuidp, locktype, &oldts,
+			      addr_uniquifier);
+	if (ts) {
+	    ReleaseWriteLock(&afs_xserver);
+	    ReleaseWriteLock(&afs_xvcb);
+	    return ts;
+	}
+    } else {
+	UpgradeSToWLock(&afs_xserver, 36);
+    }
+    ObtainWriteLock(&afs_xsrvAddr, 116);
     srvcount = afs_totalServers;
 
     /* Reuse/allocate a new server structure */
     if (oldts) {
 	newts = oldts;
     } else {
-	newts = (struct server *)afs_osi_Alloc(sizeof(struct server));
+	newts = afs_osi_Alloc(sizeof(struct server));
 	if (!newts)
 	    panic("malloc of server struct");
 	afs_totalServers++;
@@ -1830,8 +1965,6 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
     if (acell)
 	newts->cell = afs_GetCell(acell, 0);
 
-    fsport = (newts->cell ? newts->cell->fsport : AFS_FSPORT);
-
     /* For each IP address we are registering */
     for (k = 0; k < nservers; k++) {
 	iphash = SHash(aserverp[k]);
@@ -1844,7 +1977,7 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
 		break;
 	}
 	if (oldsa && (oldsa->server != newts)) {
-	    afs_RemoveSrvAddr(oldsa);	/* Remove from its server struct */
+	    afs_RemoveSrvAddr(oldsa, tv);	/* Remove from its server struct */
 	    oldsa->next_sa = newts->addr;	/* Add to the  new server struct */
 	    newts->addr = oldsa;
 	}
@@ -1853,7 +1986,7 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
 	if (oldsa) {
 	    newsa = oldsa;
 	} else {
-	    newsa = (struct srvAddr *)afs_osi_Alloc(sizeof(struct srvAddr));
+	    newsa = afs_osi_Alloc(sizeof(struct srvAddr));
 	    if (!newsa)
 		panic("malloc of srvAddr struct");
 	    afs_totalSrvAddrs++;
@@ -1903,8 +2036,7 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
 
 	    /* Have a srvAddr struct. Now get a server struct (if not already) */
 	    if (!orphts) {
-		orphts =
-		    (struct server *)afs_osi_Alloc(sizeof(struct server));
+		orphts = afs_osi_Alloc(sizeof(struct server));
 		if (!orphts)
 		    panic("malloc of lo server struct");
 		memset(orphts, 0, sizeof(struct server));
@@ -1926,7 +2058,7 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
 	    /* Hang the srvAddr struct off of the server structure. The server
 	     * may have multiple srvAddrs, but it won't be marked multihomed.
 	     */
-	    afs_RemoveSrvAddr(orphsa);	/* remove */
+	    afs_RemoveSrvAddr(orphsa, tv);	/* remove */
 	    orphsa->next_sa = orphts->addr;	/* hang off server struct */
 	    orphts->addr = orphsa;
 	    orphsa->server = orphts;
@@ -1934,6 +2066,8 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
 	    orphsa->sa_flags &= ~SRVADDR_MH;	/* Not multihomed */
 	}
     }
+    /* We can't need this below, and won't reacquire */
+    ReleaseWriteLock(&afs_xvcb);
 
     srvcount = afs_totalServers - srvcount;	/* # servers added and removed */
     if (srvcount) {
@@ -1947,6 +2081,8 @@ afs_GetServer(afs_uint32 * aserverp, afs_int32 nservers, afs_int32 acell,
 	if (afs_stats_cmperf.srvRecords > afs_stats_cmperf.srvRecordsHWM)
 	    afs_stats_cmperf.srvRecordsHWM = afs_stats_cmperf.srvRecords;
     }
+    /* We can't need this below, and won't reacquire */
+    ReleaseWriteLock(&afs_xvcb);
 
     ReleaseWriteLock(&afs_xsrvAddr);
 
@@ -2011,6 +2147,7 @@ afs_RemoveAllConns(void)
                         tc = ntc;
                     }
                     sa->conns = NULL;
+		    sa->natping = NULL;
                 }
             }
         }
