@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -31,7 +31,7 @@
 #ifdef AFS_SGI62_ENV
 #include "h/hashing.h"
 #endif
-#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN60_ENV)
+#if !defined(AFS_HPUX110_ENV) && !defined(AFS_LINUX20_ENV) && !defined(AFS_DARWIN_ENV)
 #include <netinet/in_var.h>
 #endif /* ! AFS_HPUX110_ENV */
 #endif /* !defined(UKERNEL) */
@@ -74,15 +74,15 @@ static int inVolList(struct VenusFid *fid, afs_int32 nvols, afs_int32 * vID,
 
 
 /**
- * Convert a volume name to a number; 
+ * Convert a volume name to a number;
  * @param aname Volume name.
  * @return return 0 if can't parse as a number.
  */
 static int
-afs_vtoi(register char *aname)
+afs_vtoi(char *aname)
 {
-    register afs_int32 temp;
-    register int tc;
+    afs_int32 temp;
+    int tc;
     temp = 0;
     AFS_STATCNT(afs_vtoi);
     while ((tc = *aname++)) {
@@ -106,18 +106,20 @@ afs_vtoi(register char *aname)
 static struct fvolume staticFVolume;
 afs_int32 afs_FVIndex = -1;
 
-/** 
- * UFS specific version of afs_GetVolSlot 
+/**
+ * UFS specific version of afs_GetVolSlot
  * @return
  */
 struct volume *
 afs_UFSGetVolSlot(void)
 {
-    register struct volume *tv, **lv;
+    struct volume *tv = NULL, **lv;
     struct osi_file *tfile;
-    register afs_int32 i, code;
+    afs_int32 i = -1, code;
     afs_int32 bestTime;
-    struct volume *bestVp, **bestLp;
+    struct volume *bestVp, *oldLp = NULL, **bestLp = NULL;
+    char *oldname = NULL;
+    afs_int32 oldvtix = -2; /* Initialize to a value that doesn't occur */
 
     AFS_STATCNT(afs_UFSGetVolSlot);
     if (!afs_freeVolList) {
@@ -137,13 +139,19 @@ afs_UFSGetVolSlot(void)
 		}
 	    }
 	}
-	if (!bestVp)
-	    osi_Panic("getvolslot none");
+	if (!bestVp) {
+	    afs_warn("afs_UFSGetVolSlot: no vol slots available\n");
+	    goto error;
+	}
 	tv = bestVp;
+
+	oldLp = *bestLp;
 	*bestLp = tv->next;
-	if (tv->name)
-	    afs_osi_Free(tv->name, strlen(tv->name) + 1);
+
+	oldname = tv->name;
 	tv->name = NULL;
+
+	oldvtix = tv->vtix;
 	/* now write out volume structure to file */
 	if (tv->vtix < 0) {
 	    tv->vtix = afs_volCounter++;
@@ -161,9 +169,12 @@ afs_UFSGetVolSlot(void)
 		code =
 		    afs_osi_Read(tfile, sizeof(struct fvolume) * tv->vtix,
 				 &staticFVolume, sizeof(struct fvolume));
-		if (code != sizeof(struct fvolume))
-		    osi_Panic("read volumeinfo");
 		osi_UFSClose(tfile);
+		if (code != sizeof(struct fvolume)) {
+		    afs_warn("afs_UFSGetVolSlot: error %d reading volumeinfo\n",
+		             (int)code);
+		    goto error;
+		}
 		afs_FVIndex = tv->vtix;
 	    }
 	}
@@ -178,33 +189,64 @@ afs_UFSGetVolSlot(void)
 	code =
 	    afs_osi_Write(tfile, sizeof(struct fvolume) * afs_FVIndex,
 			  &staticFVolume, sizeof(struct fvolume));
-	if (code != sizeof(struct fvolume))
-	    osi_Panic("write volumeinfo");
 	osi_UFSClose(tfile);
+	if (code != sizeof(struct fvolume)) {
+	    afs_warn("afs_UFSGetVolSlot: error %d writing volumeinfo\n",
+	             (int)code);
+	    goto error;
+	}
+	if (oldname) {
+	    afs_osi_Free(oldname, strlen(oldname) + 1);
+	    oldname = NULL;
+	}
     } else {
 	tv = afs_freeVolList;
 	afs_freeVolList = tv->next;
     }
     return tv;
 
+ error:
+    if (tv) {
+	if (oldvtix == -2) {
+	    afs_warn("afs_UFSGetVolSlot: oldvtix is uninitialized\n");
+	    return NULL;
+	}
+	if (oldname) {
+	    tv->name = oldname;
+	    oldname = NULL;
+	}
+	if (oldvtix < 0) {
+	    afs_volCounter--;
+	    fvTable[i] = staticFVolume.next;
+	}
+	if (bestLp) {
+	    *bestLp = oldLp;
+	}
+	tv->vtix = oldvtix;
+	/* we messed with staticFVolume, so make sure someone else
+	 * doesn't think it's fine to use */
+	afs_FVIndex = -1;
+    }
+    return NULL;
 }				/*afs_UFSGetVolSlot */
 
 
 /**
- *   Get an available volume list slot. If the list does not exist, 
+ *   Get an available volume list slot. If the list does not exist,
  * create one containing a single element.
- * @return 
+ * @return
  */
 struct volume *
 afs_MemGetVolSlot(void)
 {
-    register struct volume *tv;
+    struct volume *tv;
 
     AFS_STATCNT(afs_MemGetVolSlot);
     if (!afs_freeVolList) {
 	struct volume *newVp;
 
-	newVp = (struct volume *)afs_osi_Alloc(sizeof(struct volume));
+	newVp = afs_osi_Alloc(sizeof(struct volume));
+	osi_Assert(newVp != NULL);
 
 	newVp->next = NULL;
 	afs_freeVolList = newVp;
@@ -215,13 +257,17 @@ afs_MemGetVolSlot(void)
 
 }				/*afs_MemGetVolSlot */
 
-/** 
- *   Reset volume information for all volume structs that
- * point to a speicific server.
- * @param srvp
+/*!
+ * Reset volume information for all volume structs that
+ * point to a speicific server, skipping a given volume if provided.
+ *
+ * @param[in] srvp
+ *      The server to reset volume info about
+ * @param[in] tv
+ *      The volume to skip resetting info about
  */
 void
-afs_ResetVolumes(struct server *srvp)
+afs_ResetVolumes(struct server *srvp, struct volume *tv)
 {
     int j, k;
     struct volume *vp;
@@ -229,10 +275,12 @@ afs_ResetVolumes(struct server *srvp)
     /* Find any volumes residing on this server and flush their state */
     for (j = 0; j < NVOLS; j++) {
 	for (vp = afs_volumes[j]; vp; vp = vp->next) {
-	    for (k = 0; k < MAXHOSTS; k++) {
+	    for (k = 0; k < AFS_MAXHOSTS; k++) {
 		if (!srvp || (vp->serverHost[k] == srvp)) {
-		    vp->serverHost[k] = 0;
-		    afs_ResetVolumeInfo(vp);
+		    if (tv && tv != vp) {
+			vp->serverHost[k] = 0;
+			afs_ResetVolumeInfo(vp);
+		    }
 		    break;
 		}
 	    }
@@ -240,8 +288,7 @@ afs_ResetVolumes(struct server *srvp)
     }
 }
 
-
-/** 
+/**
  *   Reset volume name to volume id mapping cache.
  * @param flags
  */
@@ -271,7 +318,7 @@ afs_CheckVolumeNames(int flags)
 	    for (tv = afs_volumes[i]; tv; tv = tv->next)
 		++vsize;
 
-	volumeID = (afs_int32 *) afs_osi_Alloc(2 * vsize * sizeof(*volumeID));
+	volumeID = afs_osi_Alloc(2 * vsize * sizeof(*volumeID));
 	cellID = (volumeID) ? volumeID + vsize : 0;
     }
 
@@ -292,7 +339,7 @@ afs_CheckVolumeNames(int flags)
 	    }
 	    /* ??? */
 	    if (flags & (AFS_VOLCHECK_BUSY | AFS_VOLCHECK_FORCE)) {
-		for (j = 0; j < MAXHOSTS; j++)
+		for (j = 0; j < AFS_MAXHOSTS; j++)
 		    tv->status[j] = not_busy;
 	    }
 
@@ -413,7 +460,7 @@ inVolList(struct VenusFid *fid, afs_int32 nvols, afs_int32 * vID,
 /* afs_PutVolume is now a macro in afs.h */
 
 
-/** 
+/**
  *    Return volume struct if we have it cached and it's up-to-date.
  *  Environment: Must be called with afs_xvolume unlocked.
  *  @param afid Volume FID.
@@ -476,12 +523,12 @@ afs_GetVolume(struct VenusFid *afid, struct vrequest *areq,
 
 
 /**
- * 
+ *
  * @param volid Volume ID. If it's 0, get it from the name.
  * @param aname Volume name.
  * @param ve Volume entry.
  * @param tcell The cell containing this volume.
- * @param agood 
+ * @param agood
  * @param type Type of volume.
  * @param areq Request.
  * @return Volume or NULL if failure.
@@ -518,7 +565,7 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 		volid = nve->volumeId[whichType];
 	    } else {
 		volid = ove->volumeId[whichType];
-	    } 
+	    }
 	} /* end of if (volid == 0) */
     } /* end of if (!volid) */
 
@@ -534,6 +581,10 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
 	struct fvolume *tf = 0;
 
 	tv = afs_GetVolSlot();
+	if (!tv) {
+	    ReleaseWriteLock(&afs_xvolume);
+	    return NULL;
+	}
 	memset(tv, 0, sizeof(struct volume));
 	tv->cell = tcell->cellNum;
 	AFS_RWLOCK_INIT(&tv->lock, "volume lock");
@@ -576,16 +627,16 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
     tv->states &= ~VRecheck;	/* just checked it */
     tv->accessTime = osi_Time();
     ReleaseWriteLock(&afs_xvolume);
-    ObtainWriteLock(&tv->lock, 111);
     if (type == 2) {
-	InstallUVolumeEntry(tv, uve, tcell->cellNum, tcell, areq);
+	LockAndInstallUVolumeEntry(tv, uve, tcell->cellNum, tcell, areq);
     } else if (type == 1)
-	InstallNVolumeEntry(tv, nve, tcell->cellNum);
+	LockAndInstallNVolumeEntry(tv, nve, tcell->cellNum);
     else
-	InstallVolumeEntry(tv, ove, tcell->cellNum);
+	LockAndInstallVolumeEntry(tv, ove, tcell->cellNum);
     if (agood) {
 	if (!tv->name) {
 	    tv->name = afs_osi_Alloc(strlen(aname) + 1);
+	    osi_Assert(tv->name != NULL);
 	    strcpy(tv->name, aname);
 	}
     }
@@ -601,14 +652,14 @@ afs_SetupVolume(afs_int32 volid, char *aname, void *ve, struct cell *tcell,
  * Seek volume by it's name and attributes.
  * If volume not found, try to add one.
  * @param aname Volume name.
- * @param acell Cell 
+ * @param acell Cell
  * @param agood
  * @param areq
  * @param locktype Type of lock to be used.
- * @return 
+ * @return
  */
 struct volume *
-afs_GetVolumeByName(register char *aname, afs_int32 acell, int agood,
+afs_GetVolumeByName(char *aname, afs_int32 acell, int agood,
 		    struct vrequest *areq, afs_int32 locktype)
 {
     afs_int32 i;
@@ -638,7 +689,7 @@ afs_GetVolumeByName(register char *aname, afs_int32 acell, int agood,
 
 /**
  *   Init a new dynroot volume.
- * @param Volume FID. 
+ * @param Volume FID.
  * @return Volume or NULL if not found.
  */
 static struct volume *
@@ -653,6 +704,7 @@ afs_NewDynrootVolume(struct VenusFid *fid)
     if (!tcell)
 	return NULL;
     tve = afs_osi_Alloc(sizeof(*tve));
+    osi_Assert(tve != NULL);
     if (!(tcell->states & CHasVolRef))
 	tcell->states |= CHasVolRef;
 
@@ -673,7 +725,7 @@ int lastnvcode;
 /**
  * @param aname Volume name.
  * @param acell Cell id.
- * @param agood 
+ * @param agood
  * @param areq Request type.
  * @param locktype Type of lock to be used.
  * @return Volume or NULL if failure.
@@ -691,6 +743,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     char *tbuffer, *ve;
     struct afs_conn *tconn;
     struct vrequest treq;
+    struct rx_connection *rxconn;
 
     if (strlen(aname) > VL_MAXNAMELEN)	/* Invalid volume name */
 	return NULL;
@@ -715,34 +768,34 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
     do {
 	tconn =
 	    afs_ConnByMHosts(tcell->cellHosts, tcell->vlport, tcell->cellNum,
-			     &treq, SHARED_LOCK);
+			     &treq, SHARED_LOCK, 0, &rxconn);
 	if (tconn) {
 	    if (tconn->srvr->server->flags & SNO_LHOSTS) {
 		type = 0;
 		RX_AFS_GUNLOCK();
-		code = VL_GetEntryByNameO(tconn->id, aname, tve);
+		code = VL_GetEntryByNameO(rxconn, aname, tve);
 		RX_AFS_GLOCK();
 	    } else if (tconn->srvr->server->flags & SYES_LHOSTS) {
 		type = 1;
 		RX_AFS_GUNLOCK();
-		code = VL_GetEntryByNameN(tconn->id, aname, ntve);
+		code = VL_GetEntryByNameN(rxconn, aname, ntve);
 		RX_AFS_GLOCK();
 	    } else {
 		type = 2;
 		RX_AFS_GUNLOCK();
-		code = VL_GetEntryByNameU(tconn->id, aname, utve);
+		code = VL_GetEntryByNameU(rxconn, aname, utve);
 		RX_AFS_GLOCK();
 		if (!(tconn->srvr->server->flags & SVLSRV_UUID)) {
 		    if (code == RXGEN_OPCODE) {
 			type = 1;
 			RX_AFS_GUNLOCK();
-			code = VL_GetEntryByNameN(tconn->id, aname, ntve);
+			code = VL_GetEntryByNameN(rxconn, aname, ntve);
 			RX_AFS_GLOCK();
 			if (code == RXGEN_OPCODE) {
 			    type = 0;
 			    tconn->srvr->server->flags |= SNO_LHOSTS;
 			    RX_AFS_GUNLOCK();
-			    code = VL_GetEntryByNameO(tconn->id, aname, tve);
+			    code = VL_GetEntryByNameO(rxconn, aname, tve);
 			    RX_AFS_GLOCK();
 			} else if (!code)
 			    tconn->srvr->server->flags |= SYES_LHOSTS;
@@ -753,7 +806,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	    }
 	} else
 	    code = -1;
-    } while (afs_Analyze(tconn, code, NULL, &treq, -1,	/* no op code for this */
+    } while (afs_Analyze(tconn, rxconn, code, NULL, &treq, -1,	/* no op code for this */
 			 SHARED_LOCK, tcell));
 
     if (code) {
@@ -768,7 +821,7 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	    int i;
 	    struct server *sp;
 	    struct srvAddr *sap;
-	    for (i = 0; i < MAXCELLHOSTS; i++) {
+	    for (i = 0; i < AFS_MAXCELLHOSTS; i++) {
 		if ((sp = tcell->cellHosts[i]) == NULL)
 		    break;
 		for (sap = sp->addr; sap; sap = sap->next_sa)
@@ -798,23 +851,27 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 	ve = (char *)ntve;
     else
 	ve = (char *)tve;
-    tv = afs_SetupVolume(0, aname, ve, tcell, agood, type, areq);
-    if ((agood == 3) && tv->backVol) {
+    tv = afs_SetupVolume(0, aname, ve, tcell, agood, type, &treq);
+    if ((agood == 3) && tv && tv->backVol) {
 	/*
 	 * This means that very soon we'll ask for the BK volume so
 	 * we'll prefetch it (well we did already.)
 	 */
 	tv1 =
-	    afs_SetupVolume(tv->backVol, (char *)0, ve, tcell, 0, type, areq);
-	tv1->refCount--;
+	    afs_SetupVolume(tv->backVol, (char *)0, ve, tcell, 0, type, &treq);
+	if (tv1) {
+	    tv1->refCount--;
+	}
     }
-    if ((agood >= 2) && tv->roVol) {
+    if ((agood >= 2) && tv && tv->roVol) {
 	/*
 	 * This means that very soon we'll ask for the RO volume so
 	 * we'll prefetch it (well we did already.)
 	 */
-	tv1 = afs_SetupVolume(tv->roVol, NULL, ve, tcell, 0, type, areq);
-	tv1->refCount--;
+	tv1 = afs_SetupVolume(tv->roVol, NULL, ve, tcell, 0, type, &treq);
+	if (tv1) {
+	    tv1->refCount--;
+	}
     }
     osi_FreeLargeSpace(tbuffer);
     afs_PutCell(tcell, READ_LOCK);
@@ -824,58 +881,45 @@ afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 
 
 
-/** 
+/**
  *   Call this with the volume structure locked; used for new-style vldb requests.
- * @param av Volume 
+ * @param av Volume
  * @param ve
  * @param acell
  */
 void
-InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
+LockAndInstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 {
-    register struct server *ts;
+    struct server *ts;
     struct cell *cellp;
-    register int i, j;
+    int i, j;
     afs_int32 mask;
     afs_uint32 temp;
+    char types = 0;
+    struct server *serverHost[AFS_MAXHOSTS];
 
     AFS_STATCNT(InstallVolumeEntry);
 
+    memset(serverHost, 0, sizeof(serverHost));
+
     /* Determine the type of volume we want */
     if ((ve->flags & VLF_RWEXISTS) && (av->volume == ve->volumeId[RWVOL])) {
 	mask = VLSF_RWVOL;
     } else if ((ve->flags & VLF_ROEXISTS)
 	       && (av->volume == ve->volumeId[ROVOL])) {
 	mask = VLSF_ROVOL;
-	av->states |= VRO;
+	types |= VRO;
     } else if ((ve->flags & VLF_BACKEXISTS)
 	       && (av->volume == ve->volumeId[BACKVOL])) {
 	/* backup always is on the same volume as parent */
 	mask = VLSF_RWVOL;
-	av->states |= (VRO | VBackup);
+	types |= (VRO | VBackup);
     } else {
 	mask = 0;		/* Can't find volume in vldb entry */
     }
 
-    /* fill in volume types */
-    av->rwVol = ((ve->flags & VLF_RWEXISTS) ? ve->volumeId[RWVOL] : 0);
-    av->roVol = ((ve->flags & VLF_ROEXISTS) ? ve->volumeId[ROVOL] : 0);
-    av->backVol = ((ve->flags & VLF_BACKEXISTS) ? ve->volumeId[BACKVOL] : 0);
-
-    if (ve->flags & VLF_DFSFILESET)
-	av->states |= VForeign;
-
     cellp = afs_GetCell(acell, 0);
 
-    /* This volume, av, is locked. Zero out the serverHosts[] array 
-     * so that if afs_GetServer() decides to replace the server 
-     * struct, we don't deadlock trying to afs_ResetVolumeInfo()
-     * this volume.
-     */
-    for (j = 0; j < MAXHOSTS; j++) {
-	av->serverHost[j] = 0;
-    }
-
     /* Step through the VLDB entry making sure each server listed is there */
     for (i = 0, j = 0; i < ve->nServers; i++) {
 	if (((ve->serverFlags[i] & mask) == 0)
@@ -885,8 +929,8 @@ InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 
 	temp = htonl(ve->serverNumber[i]);
 	ts = afs_GetServer(&temp, 1, acell, cellp->fsport, WRITE_LOCK,
-			   (afsUUID *) 0, 0);
-	av->serverHost[j] = ts;
+			   (afsUUID *) 0, 0, av);
+	serverHost[j] = ts;
 
 	/*
 	 * The cell field could be 0 if the server entry was created
@@ -901,39 +945,13 @@ InstallVolumeEntry(struct volume *av, struct vldbentry *ve, int acell)
 	afs_PutServer(ts, WRITE_LOCK);
 	j++;
     }
-    if (j < MAXHOSTS) {
-	av->serverHost[j++] = 0;
-    }
-    afs_SortServers(av->serverHost, MAXHOSTS);
-}				/*InstallVolumeEntry */
 
+    ObtainWriteLock(&av->lock, 109);
 
-void
-InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
-{
-    register struct server *ts;
-    struct cell *cellp;
-    register int i, j;
-    afs_int32 mask;
-    afs_uint32 temp;
+    memcpy(av->serverHost, serverHost, sizeof(serverHost));
 
-    AFS_STATCNT(InstallVolumeEntry);
-
-    /* Determine type of volume we want */
-    if ((ve->flags & VLF_RWEXISTS) && (av->volume == ve->volumeId[RWVOL])) {
-	mask = VLSF_RWVOL;
-    } else if ((ve->flags & VLF_ROEXISTS)
-	       && (av->volume == ve->volumeId[ROVOL])) {
-	mask = VLSF_ROVOL;
-	av->states |= VRO;
-    } else if ((ve->flags & VLF_BACKEXISTS)
-	       && (av->volume == ve->volumeId[BACKVOL])) {
-	/* backup always is on the same volume as parent */
-	mask = VLSF_RWVOL;
-	av->states |= (VRO | VBackup);
-    } else {
-	mask = 0;		/* Can't find volume in vldb entry */
-    }
+    /* from above */
+    av->states |= types;
 
     /* fill in volume types */
     av->rwVol = ((ve->flags & VLF_RWEXISTS) ? ve->volumeId[RWVOL] : 0);
@@ -943,17 +961,43 @@ InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
     if (ve->flags & VLF_DFSFILESET)
 	av->states |= VForeign;
 
-    cellp = afs_GetCell(acell, 0);
+    afs_SortServers(av->serverHost, AFS_MAXHOSTS);
+}				/*InstallVolumeEntry */
 
-    /* This volume, av, is locked. Zero out the serverHosts[] array 
-     * so that if afs_GetServer() decides to replace the server 
-     * struct, we don't deadlock trying to afs_ResetVolumeInfo()
-     * this volume.
-     */
-    for (j = 0; j < MAXHOSTS; j++) {
-	av->serverHost[j] = 0;
+
+void
+LockAndInstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
+{
+    struct server *ts;
+    struct cell *cellp;
+    int i, j;
+    afs_int32 mask;
+    afs_uint32 temp;
+    char types = 0;
+    struct server *serverHost[AFS_MAXHOSTS];
+
+    AFS_STATCNT(InstallVolumeEntry);
+
+    memset(serverHost, 0, sizeof(serverHost));
+
+    /* Determine type of volume we want */
+    if ((ve->flags & VLF_RWEXISTS) && (av->volume == ve->volumeId[RWVOL])) {
+	mask = VLSF_RWVOL;
+    } else if ((ve->flags & VLF_ROEXISTS)
+	       && (av->volume == ve->volumeId[ROVOL])) {
+	mask = VLSF_ROVOL;
+	types |= VRO;
+    } else if ((ve->flags & VLF_BACKEXISTS)
+	       && (av->volume == ve->volumeId[BACKVOL])) {
+	/* backup always is on the same volume as parent */
+	mask = VLSF_RWVOL;
+	types |= (VRO | VBackup);
+    } else {
+	mask = 0;		/* Can't find volume in vldb entry */
     }
 
+    cellp = afs_GetCell(acell, 0);
+
     /* Step through the VLDB entry making sure each server listed is there */
     for (i = 0, j = 0; i < ve->nServers; i++) {
 	if (((ve->serverFlags[i] & mask) == 0)
@@ -963,8 +1007,8 @@ InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
 
 	temp = htonl(ve->serverNumber[i]);
 	ts = afs_GetServer(&temp, 1, acell, cellp->fsport, WRITE_LOCK,
-			   (afsUUID *) 0, 0);
-	av->serverHost[j] = ts;
+			   (afsUUID *) 0, 0, av);
+	serverHost[j] = ts;
 	/*
 	 * The cell field could be 0 if the server entry was created
 	 * first with the 'fs setserverprefs' call which doesn't set
@@ -978,42 +1022,13 @@ InstallNVolumeEntry(struct volume *av, struct nvldbentry *ve, int acell)
 	afs_PutServer(ts, WRITE_LOCK);
 	j++;
     }
-    if (j < MAXHOSTS) {
-	av->serverHost[j++] = 0;
-    }
-    afs_SortServers(av->serverHost, MAXHOSTS);
-}				/*InstallNVolumeEntry */
 
+    ObtainWriteLock(&av->lock, 110);
 
-void
-InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
-		    struct cell *tcell, struct vrequest *areq)
-{
-    register struct server *ts;
-    struct afs_conn *tconn;
-    struct cell *cellp;
-    register int i, j;
-    afs_uint32 serverid;
-    afs_int32 mask;
-    int k;
+    memcpy(av->serverHost, serverHost, sizeof(serverHost));
 
-    AFS_STATCNT(InstallVolumeEntry);
-
-    /* Determine type of volume we want */
-    if ((ve->flags & VLF_RWEXISTS) && (av->volume == ve->volumeId[RWVOL])) {
-	mask = VLSF_RWVOL;
-    } else if ((ve->flags & VLF_ROEXISTS)
-	       && av->volume == ve->volumeId[ROVOL]) {
-	mask = VLSF_ROVOL;
-	av->states |= VRO;
-    } else if ((ve->flags & VLF_BACKEXISTS)
-	       && (av->volume == ve->volumeId[BACKVOL])) {
-	/* backup always is on the same volume as parent */
-	mask = VLSF_RWVOL;
-	av->states |= (VRO | VBackup);
-    } else {
-	mask = 0;		/* Can't find volume in vldb entry */
-    }
+    /* from above */
+    av->states |= types;
 
     /* fill in volume types */
     av->rwVol = ((ve->flags & VLF_RWEXISTS) ? ve->volumeId[RWVOL] : 0);
@@ -1023,17 +1038,46 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
     if (ve->flags & VLF_DFSFILESET)
 	av->states |= VForeign;
 
-    cellp = afs_GetCell(acell, 0);
+    afs_SortServers(av->serverHost, AFS_MAXHOSTS);
+}				/*InstallNVolumeEntry */
 
-    /* This volume, av, is locked. Zero out the serverHosts[] array 
-     * so that if afs_GetServer() decides to replace the server 
-     * struct, we don't deadlock trying to afs_ResetVolumeInfo()
-     * this volume.
-     */
-    for (j = 0; j < MAXHOSTS; j++) {
-	av->serverHost[j] = 0;
+
+void
+LockAndInstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
+			   struct cell *tcell, struct vrequest *areq)
+{
+    struct server *ts;
+    struct afs_conn *tconn;
+    struct cell *cellp;
+    int i, j;
+    afs_uint32 serverid;
+    afs_int32 mask;
+    int k;
+    char type = 0;
+    struct server *serverHost[AFS_MAXHOSTS];
+
+    AFS_STATCNT(InstallVolumeEntry);
+
+    memset(serverHost, 0, sizeof(serverHost));
+
+    /* Determine type of volume we want */
+    if ((ve->flags & VLF_RWEXISTS) && (av->volume == ve->volumeId[RWVOL])) {
+	mask = VLSF_RWVOL;
+    } else if ((ve->flags & VLF_ROEXISTS)
+	       && av->volume == ve->volumeId[ROVOL]) {
+	mask = VLSF_ROVOL;
+	type |= VRO;
+    } else if ((ve->flags & VLF_BACKEXISTS)
+	       && (av->volume == ve->volumeId[BACKVOL])) {
+	/* backup always is on the same volume as parent */
+	mask = VLSF_RWVOL;
+	type |= (VRO | VBackup);
+    } else {
+	mask = 0;		/* Can't find volume in vldb entry */
     }
 
+    cellp = afs_GetCell(acell, 0);
+
     /* Gather the list of servers the VLDB says the volume is on
      * and initialize the ve->serverHost[] array. If a server struct
      * is not found, then get the list of addresses for the
@@ -1048,8 +1092,8 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 	if (!(ve->serverFlags[i] & VLSERVER_FLAG_UUID)) {
 	    /* The server has no uuid */
 	    serverid = htonl(ve->serverNumber[i].time_low);
-	    ts = afs_GetServer(&serverid, 1, acell, cellp->fsport, WRITE_LOCK,
-			       (afsUUID *) 0, 0);
+	    ts = afs_GetServer(&serverid, 1, acell, cellp->fsport,
+			       WRITE_LOCK, (afsUUID *) 0, 0, av);
 	} else {
 	    ts = afs_FindServer(0, cellp->fsport, &ve->serverNumber[i], 0);
 	    if (ts && (ts->sr_addr_uniquifier == ve->serverUnique[i])
@@ -1061,6 +1105,7 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		bulkaddrs addrs;
 		ListAddrByAttributes attrs;
 		afsUUID uuid;
+		struct rx_connection *rxconn;
 
 		memset(&attrs, 0, sizeof(attrs));
 		attrs.Mask = VLADDR_UUID;
@@ -1070,11 +1115,12 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		do {
 		    tconn =
 			afs_ConnByMHosts(tcell->cellHosts, tcell->vlport,
-					 tcell->cellNum, areq, SHARED_LOCK);
+					 tcell->cellNum, areq, SHARED_LOCK,
+					 0, &rxconn);
 		    if (tconn) {
 			RX_AFS_GUNLOCK();
 			code =
-			    VL_GetAddrsU(tconn->id, &attrs, &uuid, &unique,
+			    VL_GetAddrsU(rxconn, &attrs, &uuid, &unique,
 					 &nentries, &addrs);
 			RX_AFS_GLOCK();
 		    } else {
@@ -1086,7 +1132,7 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 			code = VL_NOENT;
 
 		} while (afs_Analyze
-			 (tconn, code, NULL, areq, -1, SHARED_LOCK, tcell));
+			 (tconn, rxconn, code, NULL, areq, -1, SHARED_LOCK, tcell));
 		if (code) {
 		    /* Better handing of such failures; for now we'll simply retry this call */
 		    areq->volumeError = 1;
@@ -1097,14 +1143,14 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 		for (k = 0; k < nentries; k++) {
 		    addrp[k] = htonl(addrp[k]);
 		}
-		ts = afs_GetServer(addrp, nentries, acell, cellp->fsport,
-				   WRITE_LOCK, &ve->serverNumber[i],
-				   ve->serverUnique[i]);
-		afs_osi_Free(addrs.bulkaddrs_val,
-			     addrs.bulkaddrs_len * sizeof(*addrp));
+		ts = afs_GetServer(addrp, nentries, acell,
+				   cellp->fsport, WRITE_LOCK,
+				   &ve->serverNumber[i],
+				   ve->serverUnique[i], av);
+		xdr_free((xdrproc_t) xdr_bulkaddrs, &addrs);
 	    }
 	}
-	av->serverHost[j] = ts;
+	serverHost[j] = ts;
 
 	/* The cell field could be 0 if the server entry was created
 	 * first with the 'fs setserverprefs' call which doesn't set
@@ -1119,14 +1165,29 @@ InstallUVolumeEntry(struct volume *av, struct uvldbentry *ve, int acell,
 	j++;
     }
 
-    afs_SortServers(av->serverHost, MAXHOSTS);
+    ObtainWriteLock(&av->lock, 111);
+
+    memcpy(av->serverHost, serverHost, sizeof(serverHost));
+
+    /* from above */
+    av->states |= type;
+
+    /* fill in volume types */
+    av->rwVol = ((ve->flags & VLF_RWEXISTS) ? ve->volumeId[RWVOL] : 0);
+    av->roVol = ((ve->flags & VLF_ROEXISTS) ? ve->volumeId[ROVOL] : 0);
+    av->backVol = ((ve->flags & VLF_BACKEXISTS) ? ve->volumeId[BACKVOL] : 0);
+
+    if (ve->flags & VLF_DFSFILESET)
+	av->states |= VForeign;
+
+    afs_SortServers(av->serverHost, AFS_MAXHOSTS);
 }				/*InstallVolumeEntry */
 
 
 /**
- *   Reset volume info for the specified volume strecture. Mark volume 
+ *   Reset volume info for the specified volume strecture. Mark volume
  * to be rechecked next time.
- * @param tv 
+ * @param tv
  */
 void
 afs_ResetVolumeInfo(struct volume *tv)
@@ -1136,7 +1197,12 @@ afs_ResetVolumeInfo(struct volume *tv)
     AFS_STATCNT(afs_ResetVolumeInfo);
     ObtainWriteLock(&tv->lock, 117);
     tv->states |= VRecheck;
-    for (i = 0; i < MAXHOSTS; i++)
+
+    /* the hard-mount code in afs_Analyze may not be able to reset this flag
+     * when VRecheck is set, so clear it here to ensure it gets cleared. */
+    tv->states &= ~VHardMount;
+
+    for (i = 0; i < AFS_MAXHOSTS; i++)
 	tv->status[i] = not_busy;
     if (tv->name) {
 	afs_osi_Free(tv->name, strlen(tv->name) + 1);
