@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -23,6 +23,12 @@
 #include <stdio.h>
 #include <stdlib.h>
 #include <stdarg.h>
+#ifdef HAVE_SYS_TIME_H
+#include <sys/time.h>
+#endif
+#ifdef HAVE_SYS_RESOURCE_H
+#include <sys/resource.h>
+#endif
 #ifdef AFS_NT40_ENV
 #include <winsock2.h>
 #include <direct.h>
@@ -60,9 +66,15 @@ struct afsconf_dir *bozo_confdir = 0;	/* bozo configuration dir */
 static PROCESS bozo_pid;
 const char *bozo_fileName;
 FILE *bozo_logFile;
+#ifndef AFS_NT40_ENV
+static int bozo_argc = 0;
+static char** bozo_argv = NULL;
+#endif
 
+const char *DoCore;
 int DoLogging = 0;
 int DoSyslog = 0;
+const char *DoPidFiles = NULL;
 #ifndef AFS_NT40_ENV
 int DoSyslogFacility = LOG_DAEMON;
 #endif
@@ -104,59 +116,21 @@ int
 bozo_ReBozo(void)
 {
 #ifdef AFS_NT40_ENV
-    /* exit with restart code; SCM integrator process will restart bosserver */
-    int status = BOSEXIT_RESTART;
-
-    /* if noauth flag is set, pass "-noauth" to new bosserver */
-    if (afsconf_GetNoAuthFlag(bozo_confdir)) {
-	status |= BOSEXIT_NOAUTH_FLAG;
-    }
-    /* if logging is on, pass "-log" to new bosserver */
-    if (DoLogging) {
-	status |= BOSEXIT_LOGGING_FLAG;
-    }
-    exit(status);
+    /* exit with restart code; SCM integrator process will restart bosserver with
+       the same arguments */
+    exit(BOSEXIT_RESTART);
 #else
     /* exec new bosserver process */
-    char *argv[4];
     int i = 0;
 
-    argv[i] = (char *)AFSDIR_SERVER_BOSVR_FILEPATH;
-    i++;
-
-    /* if noauth flag is set, pass "-noauth" to new bosserver */
-    if (afsconf_GetNoAuthFlag(bozo_confdir)) {
-	argv[i] = "-noauth";
-	i++;
-    }
-    /* if logging is on, pass "-log" to new bosserver */
-    if (DoLogging) {
-	argv[i] = "-log";
-	i++;
-    }
-#ifndef AFS_NT40_ENV
-    /* if syslog logging is on, pass "-syslog" to new bosserver */
-    if (DoSyslog) {
-	char *arg = (char *)malloc(40);	/* enough for -syslog=# */
-	if (DoSyslogFacility != LOG_DAEMON) {
-	    snprintf(arg, 40, "-syslog=%d", DoSyslogFacility);
-	} else {
-	    strcpy(arg, "-syslog");
-	}
-	argv[i] = arg;
-	i++;
-    }
-#endif
-
-    /* null-terminate argument list */
-    argv[i] = NULL;
-
     /* close random fd's */
     for (i = 3; i < 64; i++) {
 	close(i);
     }
 
-    execv(argv[0], argv);	/* should not return */
+    unlink(AFSDIR_SERVER_BOZRXBIND_FILEPATH);
+
+    execv(bozo_argv[0], bozo_argv);	/* should not return */
     _exit(1);
 #endif /* AFS_NT40_ENV */
 }
@@ -166,7 +140,7 @@ static int
 MakeDir(const char *adir)
 {
     struct stat tstat;
-    register afs_int32 code;
+    afs_int32 code;
     if (stat(adir, &tstat) < 0 || (tstat.st_mode & S_IFMT) != S_IFDIR) {
 	int reqPerm;
 	unlink(adir);
@@ -186,7 +160,7 @@ MakeDir(const char *adir)
 
 /* create all the bozo dirs */
 static int
-CreateDirs(void)
+CreateDirs(const char *coredir)
 {
     if ((!strncmp
 	 (AFSDIR_USR_DIRPATH, AFSDIR_CLIENT_ETC_DIRPATH,
@@ -219,14 +193,16 @@ CreateDirs(void)
     symlink(AFSDIR_SERVER_CELLSERVDB_FILEPATH,
 	    AFSDIR_CLIENT_CELLSERVDB_FILEPATH);
 #endif /* AFS_NT40_ENV */
+    if (coredir)
+	MakeDir(coredir);
     return 0;
 }
 
 /* strip the \\n from the end of the line, if it is present */
 static int
-StripLine(register char *abuffer)
+StripLine(char *abuffer)
 {
-    register char *tp;
+    char *tp;
 
     tp = abuffer + strlen(abuffer);	/* starts off pointing at the null  */
     if (tp == abuffer)
@@ -239,12 +215,12 @@ StripLine(register char *abuffer)
 
 /* write one bnode's worth of entry into the file */
 static int
-bzwrite(register struct bnode *abnode, void *arock)
+bzwrite(struct bnode *abnode, void *arock)
 {
-    register struct bztemp *at = (struct bztemp *)arock;
-    register int i;
+    struct bztemp *at = (struct bztemp *)arock;
+    int i;
     char tbuffer[BOZO_BSSIZE];
-    register afs_int32 code;
+    afs_int32 code;
 
     if (abnode->notifier)
 	fprintf(at->file, "bnode %s %s %d %s\n", abnode->type->name,
@@ -269,15 +245,16 @@ bzwrite(register struct bnode *abnode, void *arock)
 int
 ReadBozoFile(char *aname)
 {
-    register FILE *tfile;
+    FILE *tfile;
     char tbuffer[BOZO_BSSIZE];
-    register char *tp;
+    char *tp;
     char *instp, *typep, *notifier, *notp;
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 ktmask, ktday, kthour, ktmin, ktsec;
     afs_int32 i, goal;
     struct bnode *tb;
     char *parms[MAXPARMS];
+    char *thisparms[MAXPARMS];
     int rmode;
 
     /* rename BozoInit to BosServer for the user */
@@ -393,6 +370,8 @@ ReadBozoFile(char *aname)
 	} else if (code == 3)
 	    notifier = NULL;
 
+	memset(thisparms, 0, sizeof(thisparms));
+
 	for (i = 0; i < MAXPARMS; i++) {
 	    /* now read the parms, until we see an "end" line */
 	    tp = fgets(tbuffer, sizeof(tbuffer), tfile);
@@ -410,12 +389,13 @@ ReadBozoFile(char *aname)
 	    if (!parms[i])	/* make sure there's space */
 		parms[i] = (char *)malloc(BOZO_BSSIZE);
 	    strcpy(parms[i], tbuffer + 5);	/* remember the parameter for later */
+	    thisparms[i] = parms[i];
 	}
 
 	/* ok, we have the type and parms, now create the object */
 	code =
-	    bnode_Create(typep, instp, &tb, parms[0], parms[1], parms[2],
-			 parms[3], parms[4], notifier,
+	    bnode_Create(typep, instp, &tb, thisparms[0], thisparms[1],
+			 thisparms[2], thisparms[3], thisparms[4], notifier,
 			 goal ? BSTAT_NORMAL : BSTAT_SHUTDOWN, 0);
 	if (code)
 	    goto fail;
@@ -449,9 +429,9 @@ ReadBozoFile(char *aname)
 int
 WriteBozoFile(char *aname)
 {
-    register FILE *tfile;
+    FILE *tfile;
     char tbuffer[AFSDIR_PATH_MAX];
-    register afs_int32 code;
+    afs_int32 code;
     struct bztemp btemp;
 
     if (!aname)
@@ -490,9 +470,9 @@ WriteBozoFile(char *aname)
 }
 
 static int
-bdrestart(register struct bnode *abnode, void *arock)
+bdrestart(struct bnode *abnode, void *arock)
 {
-    register afs_int32 code;
+    afs_int32 code;
 
     if (abnode->fileGoal != BSTAT_NORMAL || abnode->goal != BSTAT_NORMAL)
 	return 0;		/* don't restart stopped bnodes */
@@ -513,7 +493,7 @@ bdrestart(register struct bnode *abnode, void *arock)
 static void *
 BozoDaemon(void *unused)
 {
-    register afs_int32 now;
+    afs_int32 now;
 
     /* now initialize the values */
     bozo_newKTs = 1;
@@ -592,7 +572,7 @@ tweak_config(void)
  * fundamental errors occur.
  *
  * This routine requires
- * 
+ *
  * #include <sys/types.h>
  * #include <sys/stat.h>
  * #include <fcntl.h>
@@ -614,7 +594,7 @@ tweak_config(void)
 static void
 background(void)
 {
-    /* 
+    /*
      * A process is a process group leader if its process ID
      * (getpid()) and its process group ID (getpgrp()) are the same.
      */
@@ -705,6 +685,119 @@ background(void)
 #endif /* ! AFS_NT40_ENV */
 #endif
 
+static char *
+make_pid_filename(char *ainst, char *aname)
+{
+    char *buffer = NULL;
+    int length;
+
+    length = strlen(DoPidFiles) + strlen(ainst) + 6;
+    if (aname && *aname) {
+	length += strlen(aname) + 1;
+    }
+    buffer = malloc(length * sizeof(char));
+    if (!buffer) {
+	if (aname) {
+	    bozo_Log("Failed to alloc pid filename buffer for %s.%s.\n",
+		     ainst, aname);
+	} else {
+	    bozo_Log("Failed to alloc pid filename buffer for %s.\n", ainst);
+	}
+    } else {
+	if (aname && *aname) {
+	    snprintf(buffer, length, "%s/%s.%s.pid", DoPidFiles, ainst,
+		     aname);
+	} else {
+	    snprintf(buffer, length, "%s/%s.pid", DoPidFiles, ainst);
+	}
+    }
+    return buffer;
+}
+
+/**
+ * Write a file containing the pid of the named process.
+ *
+ * @param ainst instance name
+ * @param aname sub-process name of the instance, may be null
+ * @param apid  process id of the newly started process
+ *
+ * @returns status
+ */
+int
+bozo_CreatePidFile(char *ainst, char *aname, pid_t apid)
+{
+    int code = 0;
+    char *pidfile = NULL;
+    FILE *fp;
+
+    pidfile = make_pid_filename(ainst, aname);
+    if (!pidfile) {
+	return ENOMEM;
+    }
+    if ((fp = fopen(pidfile, "w")) == NULL) {
+	bozo_Log("Failed to open pidfile %s; errno=%d\n", pidfile, errno);
+	free(pidfile);
+	return errno;
+    }
+    if (fprintf(fp, "%ld\n", afs_printable_int32_ld(apid)) < 0) {
+	code = errno;
+    }
+    if (fclose(fp) != 0) {
+	code = errno;
+    }
+    free(pidfile);
+    return code;
+}
+
+/**
+ * Clean a pid file for a process which just exited.
+ *
+ * @param ainst instance name
+ * @param aname sub-process name of the instance, may be null
+ *
+ * @returns status
+ */
+int
+bozo_DeletePidFile(char *ainst, char *aname)
+{
+    char *pidfile = NULL;
+    pidfile = make_pid_filename(ainst, aname);
+    if (pidfile) {
+	unlink(pidfile);
+	free(pidfile);
+    }
+    return 0;
+}
+
+/**
+ * Create the rxbind file of this bosserver.
+ *
+ * @param host  bind address of this server
+ *
+ * @returns status
+ */
+void
+bozo_CreateRxBindFile(afs_uint32 host)
+{
+    char buffer[16];
+    FILE *fp;
+
+    afs_inet_ntoa_r(host, buffer);
+    bozo_Log("Listening on %s:%d\n", buffer, AFSCONF_NANNYPORT);
+    if ((fp = fopen(AFSDIR_SERVER_BOZRXBIND_FILEPATH, "w")) == NULL) {
+	bozo_Log("Unable to open rxbind address file: %s, code=%d\n",
+		 AFSDIR_SERVER_BOZRXBIND_FILEPATH, errno);
+    } else {
+	/* If listening on any interface, write the loopback interface
+	   to the rxbind file to give local scripts a usable addresss. */
+	if (host == htonl(INADDR_ANY)) {
+	    afs_inet_ntoa_r(htonl(0x7f000001), buffer);
+	}
+	fprintf(fp, "%s\n", buffer);
+	fclose(fp);
+    }
+}
+
 /* start a process and monitor it */
 
 #include "AFS_component_version_number.c"
@@ -713,7 +806,7 @@ int
 main(int argc, char **argv, char **envp)
 {
     struct rx_service *tservice;
-    register afs_int32 code;
+    afs_int32 code;
     struct afsconf_dir *tdir;
     int noAuth = 0;
     int i;
@@ -731,14 +824,14 @@ main(int argc, char **argv, char **envp)
     struct sigaction nsa;
 
     /* for some reason, this permits user-mode RX to run a lot faster.
-     * we do it here in the bosserver, so we don't have to do it 
+     * we do it here in the bosserver, so we don't have to do it
      * individually in each server.
      */
     tweak_config();
 
     /*
-     * The following signal action for AIX is necessary so that in case of a 
-     * crash (i.e. core is generated) we can include the user's data section 
+     * The following signal action for AIX is necessary so that in case of a
+     * crash (i.e. core is generated) we can include the user's data section
      * in the core dump. Unfortunately, by default, only a partial core is
      * generated which, in many cases, isn't too useful.
      */
@@ -772,6 +865,7 @@ main(int argc, char **argv, char **envp)
 
     /* some path inits */
     bozo_fileName = AFSDIR_SERVER_BOZCONF_FILEPATH;
+    DoCore = AFSDIR_SERVER_LOGS_DIRPATH;
 
     /* initialize the list of dirpaths that the bosserver has
      * an interest in monitoring */
@@ -785,8 +879,23 @@ main(int argc, char **argv, char **envp)
     }
 #endif
 
+#ifndef AFS_NT40_ENV
+    /* save args for restart */
+    bozo_argc = argc;
+    bozo_argv = malloc((argc+1) * sizeof(char*));
+    if (!bozo_argv) {
+	fprintf(stderr, "%s: Failed to allocate argument list.\n", argv[0]);
+	exit(1);
+    }
+    bozo_argv[0] = (char*)AFSDIR_SERVER_BOSVR_FILEPATH; /* expected path */
+    bozo_argv[bozo_argc] = NULL; /* null terminate list */
+#endif	/* AFS_NT40_ENV */
+
     /* parse cmd line */
     for (code = 1; code < argc; code++) {
+#ifndef AFS_NT40_ENV
+	bozo_argv[code] = argv[code];
+#endif	/* AFS_NT40_ENV */
 	if (strcmp(argv[code], "-noauth") == 0) {
 	    /* set noauth flag */
 	    noAuth = 1;
@@ -801,6 +910,11 @@ main(int argc, char **argv, char **envp)
 	} else if (strncmp(argv[code], "-syslog=", 8) == 0) {
 	    DoSyslog = 1;
 	    DoSyslogFacility = atoi(argv[code] + 8);
+	} else if (strncmp(argv[code], "-cores=", 7) == 0) {
+	    if (strcmp((argv[code]+7), "none") == 0)
+		DoCore = 0;
+	    else
+		DoCore = (argv[code]+7);
 	} else if (strcmp(argv[code], "-nofork") == 0) {
 	    nofork = 1;
 	}
@@ -821,14 +935,14 @@ main(int argc, char **argv, char **envp)
 	}
 	else if (!strcmp(argv[code], "-rxmaxmtu")) {
 	    if ((code + 1) >= argc) {
-		fprintf(stderr, "missing argument for -rxmaxmtu\n"); 
-		exit(1); 
+		fprintf(stderr, "missing argument for -rxmaxmtu\n");
+		exit(1);
 	    }
 	    rxMaxMTU = atoi(argv[++code]);
-	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) || 
+	    if ((rxMaxMTU < RX_MIN_PACKET_SIZE) ||
 		(rxMaxMTU > RX_MAX_PACKET_DATA_SIZE)) {
 		printf("rxMaxMTU %d invalid; must be between %d-%" AFS_SIZET_FMT "\n",
-			rxMaxMTU, RX_MIN_PACKET_SIZE, 
+			rxMaxMTU, RX_MIN_PACKET_SIZE,
 			RX_MAX_PACKET_DATA_SIZE);
 		exit(1);
 	    }
@@ -843,6 +957,10 @@ main(int argc, char **argv, char **envp)
 		printf("Invalid audit interface '%s'\n", interface);
 		exit(1);
 	    }
+	} else if (strncmp(argv[code], "-pidfiles=", 10) == 0) {
+	    DoPidFiles = (argv[code]+10);
+	} else if (strncmp(argv[code], "-pidfiles", 9) == 0) {
+	    DoPidFiles = AFSDIR_BOSCONFIG_DIR;
 	}
 	else {
 
@@ -854,14 +972,20 @@ main(int argc, char **argv, char **envp)
 		   "[-audit-interafce <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
 		   "[-syslog[=FACILITY]] "
+		   "[-restricted] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-cores=<none|path>] \n"
+		   "[-pidfiles[=path]] "
 		   "[-nofork] " "[-help]\n");
 #else
 	    printf("Usage: bosserver [-noauth] [-log] "
 		   "[-auditlog <log path>] "
 		   "[-audit-interafce <file|sysvmq> (default is file)] "
 		   "[-rxmaxmtu <bytes>] [-rxbind] [-allow-dotted-principals]"
+		   "[-restricted] "
 		   "[-enable_peer_stats] [-enable_process_stats] "
+		   "[-cores=<none|path>] \n"
+		   "[-pidfiles[=path]] "
 		   "[-help]\n");
 #endif
 	    fflush(stdout);
@@ -892,17 +1016,20 @@ main(int argc, char **argv, char **envp)
     bnode_Register("cron", &cronbnode_ops, 2);
 
     /* create useful dirs */
-    CreateDirs();
+    CreateDirs(DoCore);
 
     /* chdir to AFS log directory */
-    chdir(AFSDIR_SERVER_LOGS_DIRPATH);
+    if (DoCore)
+	chdir(DoCore);
+    else
+	chdir(AFSDIR_SERVER_LOGS_DIRPATH);
 
 #if 0
     fputs(AFS_GOVERNMENT_MESSAGE, stdout);
     fflush(stdout);
 #endif
 
-    /* go into the background and remove our controlling tty, close open 
+    /* go into the background and remove our controlling tty, close open
        file desriptors
      */
 
@@ -913,7 +1040,7 @@ main(int argc, char **argv, char **envp)
 
     if ((!DoSyslog)
 #ifndef AFS_NT40_ENV
-	&& ((lstat(AFSDIR_BOZLOG_FILE, &sb) == 0) && 
+	&& ((lstat(AFSDIR_BOZLOG_FILE, &sb) == 0) &&
 	!(S_ISFIFO(sb.st_mode)))
 #endif
 	) {
@@ -934,11 +1061,45 @@ main(int argc, char **argv, char **envp)
 #endif
     }
 
+#if defined(RLIMIT_CORE) && defined(HAVE_GETRLIMIT)
+    {
+      struct rlimit rlp;
+      getrlimit(RLIMIT_CORE, &rlp);
+      if (!DoCore)
+	  rlp.rlim_cur = 0;
+      else
+	  rlp.rlim_max = rlp.rlim_cur = RLIM_INFINITY;
+      setrlimit(RLIMIT_CORE, &rlp);
+      getrlimit(RLIMIT_CORE, &rlp);
+      bozo_Log("Core limits now %d %d\n",(int)rlp.rlim_cur,(int)rlp.rlim_max);
+    }
+#endif
+
     /* Write current state of directory permissions to log file */
     DirAccessOK();
 
+    if (rxBind) {
+	afs_int32 ccode;
+	if (AFSDIR_SERVER_NETRESTRICT_FILEPATH ||
+	    AFSDIR_SERVER_NETINFO_FILEPATH) {
+	    char reason[1024];
+	    ccode = parseNetFiles(SHostAddrs, NULL, NULL,
+	                          ADDRSPERSITE, reason,
+	                          AFSDIR_SERVER_NETINFO_FILEPATH,
+	                          AFSDIR_SERVER_NETRESTRICT_FILEPATH);
+        } else {
+            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
+        }
+        if (ccode == 1)
+            host = SHostAddrs[0];
+    }
+
     for (i = 0; i < 10; i++) {
-	code = rx_Init(htons(AFSCONF_NANNYPORT));
+	if (rxBind) {
+	    code = rx_InitHost(host, htons(AFSCONF_NANNYPORT));
+	} else {
+	    code = rx_Init(htons(AFSCONF_NANNYPORT));
+	}
 	if (code) {
 	    bozo_Log("can't initialize rx: code=%d\n", code);
 	    sleep(3);
@@ -997,6 +1158,8 @@ main(int argc, char **argv, char **envp)
 	exit(code);
     }
 
+    bozo_CreateRxBindFile(host);	/* for local scripts */
+
     /* opened the cell databse */
     bozo_confdir = tdir;
 
@@ -1006,6 +1169,10 @@ main(int argc, char **argv, char **envp)
     afsconf_SetNoAuthFlag(tdir, noAuth);
     afsconf_BuildServerSecurityObjects(tdir, 0, &securityClasses, &numClasses);
 
+    if (DoPidFiles) {
+	bozo_CreatePidFile("bosserver", NULL, getpid());
+    }
+
     /* Disable jumbograms */
     rx_SetNoJumbo();
 
@@ -1013,23 +1180,6 @@ main(int argc, char **argv, char **envp)
 	rx_SetMaxMTU(rxMaxMTU);
     }
 
-    if (rxBind) {
-	afs_int32 ccode;
-        if (AFSDIR_SERVER_NETRESTRICT_FILEPATH || 
-            AFSDIR_SERVER_NETINFO_FILEPATH) {
-            char reason[1024];
-            ccode = parseNetFiles(SHostAddrs, NULL, NULL,
-                                           ADDRSPERSITE, reason,
-                                           AFSDIR_SERVER_NETINFO_FILEPATH,
-                                           AFSDIR_SERVER_NETRESTRICT_FILEPATH);
-        } else 
-	{
-            ccode = rx_getAllAddr(SHostAddrs, ADDRSPERSITE);
-        }
-        if (ccode == 1) 
-            host = SHostAddrs[0];
-    }
-
     tservice = rx_NewServiceHost(host, 0, /* service id */ 1,
 			         "bozo", securityClasses, numClasses,
 				 BOZO_ExecuteRequest);
