@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -26,19 +26,23 @@
 #include "kautils.h"
 #include "kaserver.h"
 
+#if !defined(offsetof)
+#include <stddef.h>             /* for definition of offsetof() */
+#endif
+
 extern Date cheaderReadTime;	/* time cheader last read in */
 
-#define set_header_word(tt,field,value) kawrite ((tt), ((char *)&(cheader.field) - (char *)&cheader), ((cheader.field = (value)), (char *)&(cheader.field)), sizeof(afs_int32))
+#define set_header_word(tt,field,value) kawrite ((tt), (offsetof(struct kaheader, field)), ((cheader.field = (value)), (char *)&(cheader.field)), sizeof(afs_int32))
 
-#define inc_header_word(tt,field) kawrite ((tt), ((char *)&(cheader.field) - (char *)&cheader), ((cheader.field = (htonl(ntohl(cheader.field)+1))), (char *)&(cheader.field)), sizeof(afs_int32))
+#define inc_header_word(tt,field) kawrite ((tt), (offsetof(struct kaheader, field)), ((cheader.field = (htonl(ntohl(cheader.field)+1))), (char *)&(cheader.field)), sizeof(afs_int32))
 
 static int index_OK(afs_int32);
 
 afs_int32
-NameHash(register char *aname, register char *ainstance)
+NameHash(char *aname, char *ainstance)
 {
-    register unsigned int hash;
-    register int i;
+    unsigned int hash;
+    int i;
 
     /* stolen directly from the HashString function in the vol package */
     hash = 0;
@@ -77,7 +81,6 @@ karead(struct ubik_trans *tt, afs_int32 pos, char *buff, afs_int32 len)
     return code;
 }
 
-static struct Lock cheader_lock;
 static struct Lock keycache_lock;
 
 static int maxCachedKeys;
@@ -98,7 +101,6 @@ static int dbfixup = 0;
 void
 init_kadatabase(int initFlags)
 {
-    Lock_Init(&cheader_lock);
     Lock_Init(&keycache_lock);
 
     maxCachedKeys = 10;
@@ -116,20 +118,25 @@ init_kadatabase(int initFlags)
 
 /* check that the database has been initialized.  Be careful to fail in a safe
    manner, to avoid bogusly reinitializing the db.  */
-
-afs_int32
-CheckInit(struct ubik_trans *at,
-          int (*db_init) (struct ubik_trans *))		/* procedure to call if rebuilding DB */
+/**
+ * reads in db cache from ubik.
+ *
+ * @param[in] ut ubik transaction
+ * @param[in] rock  opaque pointer to an int (*) (struct ubik_trans *), which
+ *                  will be called on rebuilding the database (or NULL to not
+ *                  rebuild the db)
+ *
+ * @return operation status
+ *   @retval 0 success
+ */
+static afs_int32
+UpdateCache(struct ubik_trans *at, void *rock)
 {
-    register afs_int32 code;
+    int (*db_init) (struct ubik_trans *) = rock;
+    afs_int32 code;
     afs_int32 iversion;
     afs_int32 tversion;
 
-    /* Don't read header if not necessary */
-    if (!ubik_CacheUpdate(at))
-	return 0;
-
-    ObtainWriteLock(&cheader_lock);
     if ((code = karead(at, 0, (char *)&iversion, sizeof(iversion)))
 	|| (code =
 	    karead(at, sizeof(cheader) - sizeof(afs_int32), (char *)&tversion,
@@ -155,7 +162,6 @@ CheckInit(struct ubik_trans *at,
 	    code = KAIO;
 	}
     }
-    ReleaseWriteLock(&cheader_lock);
     if (code == 0)
 	return 0;
 
@@ -190,13 +196,20 @@ CheckInit(struct ubik_trans *at,
     return db_init(at);		/* initialize the db */
 }
 
+afs_int32
+CheckInit(struct ubik_trans *at,
+          int (*db_init) (struct ubik_trans *))		/* procedure to call if rebuilding DB */
+{
+    return ubik_CheckCache(at, UpdateCache, db_init);
+}
+
 /* Allocate a free block of storage for entry, returning address of a new
    zeroed entry.  If zero is returned, a Ubik I/O error can be assumed.  */
 
 afs_int32
 AllocBlock(struct ubik_trans *at, struct kaentry *tentry)
 {
-    register afs_int32 code;
+    afs_int32 code;
     afs_int32 temp;
 
     if (cheader.freePtr) {
@@ -256,11 +269,11 @@ FreeBlock(struct ubik_trans *at, afs_int32 index)
    returned. */
 
 afs_int32
-FindBlock(struct ubik_trans *at, char *aname, char *ainstance, afs_int32 *toP, 
+FindBlock(struct ubik_trans *at, char *aname, char *ainstance, afs_int32 *toP,
 	  struct kaentry *tentry)
 {
-    register afs_int32 i, code;
-    register afs_int32 to;
+    afs_int32 i, code;
+    afs_int32 to;
 
     *toP = 0;
     i = NameHash(aname, ainstance);
@@ -286,7 +299,7 @@ FindBlock(struct ubik_trans *at, char *aname, char *ainstance, afs_int32 *toP,
    returns zero if there were no errors. */
 
 afs_int32
-ThreadBlock(struct ubik_trans *at, afs_int32 index, 
+ThreadBlock(struct ubik_trans *at, afs_int32 index,
 	    struct kaentry *tentry)
 {
     int code;
@@ -311,8 +324,8 @@ ThreadBlock(struct ubik_trans *at, afs_int32 index,
 afs_int32
 UnthreadBlock(struct ubik_trans *at, struct kaentry *aentry)
 {
-    register afs_int32 i, code;
-    register afs_int32 to;
+    afs_int32 i, code;
+    afs_int32 to;
     afs_int32 lo;
     struct kaentry tentry;
 
@@ -355,7 +368,7 @@ UnthreadBlock(struct ubik_trans *at, struct kaentry *aentry)
    remaining count is negative.  */
 
 afs_int32
-NextBlock(struct ubik_trans *at, afs_int32 index, struct kaentry *tentry, 
+NextBlock(struct ubik_trans *at, afs_int32 index, struct kaentry *tentry,
 	  afs_int32 *remaining)
 {
     int code;
@@ -392,7 +405,7 @@ NextBlock(struct ubik_trans *at, afs_int32 index, struct kaentry *tentry,
    and pointer to the user entry. */
 
 afs_int32
-ka_NewKey(struct ubik_trans *tt, afs_int32 tentryaddr, 
+ka_NewKey(struct ubik_trans *tt, afs_int32 tentryaddr,
 	  struct kaentry *tentry, struct ktc_encryptionKey *key)
 {
     struct kaOldKeys okeys;	/* old keys block */
@@ -402,7 +415,11 @@ ka_NewKey(struct ubik_trans *tt, afs_int32 tentryaddr,
     Date now = time(0);
     afs_int32 newkeyver;	/* new key version number */
     afs_int32 newtotalkeyentries = 0, oldtotalkeyentries = 0, keyentries;
-    int foundcurrentkey = 0, addednewkey = 0, modified;
+    int addednewkey = 0, modified;
+#ifdef AUTH_DBM_LOG
+    int foundcurrentkey = 0;
+#endif
+
 
     es_Report("Newkey for %s.%s\n", tentry->userID.name,
 	      tentry->userID.instance);
@@ -451,8 +468,8 @@ ka_NewKey(struct ubik_trans *tt, afs_int32 tentryaddr,
 			    ("Warning: Entry %s.%s contains more than one valid key: fixing\n",
 			     tentry->userID.name, tentry->userID.instance));
 		}
-#endif
 		foundcurrentkey = 1;
+#endif
 	    }
 
 	    /* If we find an oldkey of the same version or
@@ -566,7 +583,7 @@ ka_NewKey(struct ubik_trans *tt, afs_int32 tentryaddr,
 }
 
 afs_int32
-ka_DelKey(struct ubik_trans *tt, afs_int32 tentryaddr, 
+ka_DelKey(struct ubik_trans *tt, afs_int32 tentryaddr,
 	  struct kaentry *tentry)
 {
     int code;
@@ -628,7 +645,8 @@ ka_debugKeyCache(struct ka_debugInfo *info)
 {
     int i;
 
-    memcpy(&info->cheader_lock, &cheader_lock, sizeof(info->cheader_lock));
+    /* cheader_lock no longer exists */
+    memset(&info->cheader_lock, 0, sizeof(info->cheader_lock));
     memcpy(&info->keycache_lock, &keycache_lock, sizeof(info->keycache_lock));
 
     info->kcVersion = keyCacheVersion;
@@ -668,7 +686,7 @@ ka_debugKeyCache(struct ka_debugInfo *info)
 /* Add a key to the key cache, expanding it if necessary. */
 
 void
-ka_Encache(char *name, char *inst, afs_int32 kvno, 
+ka_Encache(char *name, char *inst, afs_int32 kvno,
 	   struct ktc_encryptionKey *key, Date superseded)
 {
     int i;
@@ -717,7 +735,7 @@ ka_Encache(char *name, char *inst, afs_int32 kvno,
    with tt==0, since Rx can't call Ubik. */
 
 afs_int32
-ka_LookupKvno(struct ubik_trans *tt, char *name, char *inst, afs_int32 kvno, 
+ka_LookupKvno(struct ubik_trans *tt, char *name, char *inst, afs_int32 kvno,
 	      struct ktc_encryptionKey *key)
 {
     int i;
@@ -782,9 +800,9 @@ ka_LookupKvno(struct ubik_trans *tt, char *name, char *inst, afs_int32 kvno,
 /* Look up the primary key and key version for a principal. */
 
 afs_int32
-ka_LookupKey(struct ubik_trans *tt, 
-	     char *name, 
-	     char *inst, 
+ka_LookupKey(struct ubik_trans *tt,
+	     char *name,
+	     char *inst,
 	     afs_int32 *kvno, 			/* returned */
 	     struct ktc_encryptionKey *key)	/* copied out */
 {
