@@ -1,7 +1,7 @@
 /*
  * Copyright 2000, International Business Machines Corporation and others.
  * All Rights Reserved.
- * 
+ *
  * This software has been released under the terms of the IBM Public
  * License.  For details, see the LICENSE file in the top-level source
  * directory or online at http://www.openafs.org/dl/license10.html
@@ -38,11 +38,7 @@
 #include "nfs.h"
 #include "ihandle.h"
 #include "viceinode.h"
-#ifdef AFS_PTHREAD_ENV
-#include <assert.h>
-#else /* AFS_PTHREAD_ENV */
-#include "afs/assert.h"
-#endif /* AFS_PTHREAD_ENV */
+#include "afs/afs_assert.h"
 #include <limits.h>
 
 #ifndef AFS_NT40_ENV
@@ -125,7 +121,7 @@ void ih_PkgDefaults(void)
 void
 ih_glock_init(void)
 {
-    assert(pthread_mutex_init(&ih_glock_mutex, NULL) == 0);
+    MUTEX_INIT(&ih_glock_mutex, "ih glock", MUTEX_DEFAULT, 0);
 }
 #endif /* AFS_PTHREAD_ENV */
 
@@ -134,7 +130,7 @@ void
 ih_Initialize(void)
 {
     int i;
-    assert(!ih_Inited);
+    osi_Assert(!ih_Inited);
     ih_Inited = 1;
     DLL_INIT_LIST(ihAvailHead, ihAvailTail);
     DLL_INIT_LIST(fdAvailHead, fdAvailTail);
@@ -147,9 +143,9 @@ ih_Initialize(void)
 #elif defined(AFS_SUN5_ENV) || defined(AFS_NBSD_ENV)
     {
 	struct rlimit rlim;
-	assert(getrlimit(RLIMIT_NOFILE, &rlim) == 0);
+	osi_Assert(getrlimit(RLIMIT_NOFILE, &rlim) == 0);
 	rlim.rlim_cur = rlim.rlim_max;
-	assert(setrlimit(RLIMIT_NOFILE, &rlim) == 0);
+	osi_Assert(setrlimit(RLIMIT_NOFILE, &rlim) == 0);
 	fdMaxCacheSize = rlim.rlim_cur - vol_io_params.fd_handle_setaside;
 #ifdef AFS_NBSD_ENV
 	/* XXX this is to avoid using up all system fd netbsd is
@@ -163,7 +159,7 @@ ih_Initialize(void)
 	fdMaxCacheSize /= 4;
 #endif
 	fdMaxCacheSize = MIN(fdMaxCacheSize, vol_io_params.fd_max_cachesize);
-	assert(fdMaxCacheSize > 0);
+	osi_Assert(fdMaxCacheSize > 0);
     }
 #elif defined(AFS_HPUX_ENV)
     /* Avoid problems with "UFSOpen: igetinode failed" panics on HPUX 11.0 */
@@ -225,9 +221,9 @@ iHandleAllocateChunk(void)
     int i;
     IHandle_t *ihP;
 
-    assert(ihAvailHead == NULL);
+    osi_Assert(ihAvailHead == NULL);
     ihP = (IHandle_t *) malloc(I_HANDLE_MALLOCSIZE * sizeof(IHandle_t));
-    assert(ihP != NULL);
+    osi_Assert(ihP != NULL);
     for (i = 0; i < I_HANDLE_MALLOCSIZE; i++) {
 	ihP[i].ih_refcnt = 0;
 	DLL_INSERT_TAIL(&ihP[i], ihAvailHead, ihAvailTail, ih_next, ih_prev);
@@ -264,12 +260,13 @@ ih_init(int dev, int vid, Inode ino)
 	iHandleAllocateChunk();
     }
     ihP = ihAvailHead;
-    assert(ihP->ih_refcnt == 0);
+    osi_Assert(ihP->ih_refcnt == 0);
     DLL_DELETE(ihP, ihAvailHead, ihAvailTail, ih_next, ih_prev);
     ihP->ih_dev = dev;
     ihP->ih_vid = vid;
     ihP->ih_ino = ino;
     ihP->ih_flags = 0;
+    ihP->ih_synced = 0;
     ihP->ih_refcnt = 1;
     DLL_INIT_LIST(ihP->ih_fdhead, ihP->ih_fdtail);
     DLL_INSERT_TAIL(ihP, ihashTable[ihash].ihash_head,
@@ -283,8 +280,8 @@ IHandle_t *
 ih_copy(IHandle_t * ihP)
 {
     IH_LOCK;
-    assert(ih_Inited);
-    assert(ihP->ih_refcnt > 0);
+    osi_Assert(ih_Inited);
+    osi_Assert(ihP->ih_refcnt > 0);
     ihP->ih_refcnt++;
     IH_UNLOCK;
     return ihP;
@@ -297,13 +294,16 @@ fdHandleAllocateChunk(void)
     int i;
     FdHandle_t *fdP;
 
-    assert(fdAvailHead == NULL);
+    osi_Assert(fdAvailHead == NULL);
     fdP = (FdHandle_t *) malloc(FD_HANDLE_MALLOCSIZE * sizeof(FdHandle_t));
-    assert(fdP != NULL);
+    osi_Assert(fdP != NULL);
     for (i = 0; i < FD_HANDLE_MALLOCSIZE; i++) {
 	fdP[i].fd_status = FD_HANDLE_AVAIL;
+	fdP[i].fd_refcnt = 0;
 	fdP[i].fd_ih = NULL;
 	fdP[i].fd_fd = INVALID_FD;
+        fdP[i].fd_ihnext = NULL;
+        fdP[i].fd_ihprev = NULL;
 	DLL_INSERT_TAIL(&fdP[i], fdAvailHead, fdAvailTail, fd_next, fd_prev);
     }
 }
@@ -315,10 +315,10 @@ streamHandleAllocateChunk(void)
     int i;
     StreamHandle_t *streamP;
 
-    assert(streamAvailHead == NULL);
+    osi_Assert(streamAvailHead == NULL);
     streamP = (StreamHandle_t *)
 	malloc(STREAM_HANDLE_MALLOCSIZE * sizeof(StreamHandle_t));
-    assert(streamP != NULL);
+    osi_Assert(streamP != NULL);
     for (i = 0; i < STREAM_HANDLE_MALLOCSIZE; i++) {
 	streamP[i].str_fd = INVALID_FD;
 	DLL_INSERT_TAIL(&streamP[i], streamAvailHead, streamAvailTail,
@@ -343,15 +343,32 @@ ih_open(IHandle_t * ihP)
 
     /* Do we already have an open file handle for this Inode? */
     for (fdP = ihP->ih_fdtail; fdP != NULL; fdP = fdP->fd_ihprev) {
-	if (fdP->fd_status != FD_HANDLE_INUSE) {
-	    assert(fdP->fd_status == FD_HANDLE_OPEN);
+	if (fdP->fd_status == FD_HANDLE_CLOSING) {
+	    /* The handle was open when an IH_REALLYCLOSE was issued, so we
+	     * cannot reuse it; it will be closed soon. */
+	    continue;
+	}
+#ifndef HAVE_PIO
+	/*
+	 * If we don't have positional i/o, don't try to share fds, since
+	 * we can't do so in a threadsafe way.
+	 */
+	if (fdP->fd_status == FD_HANDLE_INUSE) {
+	    continue;
+	}
+	osi_Assert(fdP->fd_status == FD_HANDLE_OPEN);
+#else /* HAVE_PIO */
+	osi_Assert(fdP->fd_status != FD_HANDLE_AVAIL);
+#endif /* HAVE_PIO */
+
+	fdP->fd_refcnt++;
+	if (fdP->fd_status == FD_HANDLE_OPEN) {
 	    fdP->fd_status = FD_HANDLE_INUSE;
 	    DLL_DELETE(fdP, fdLruHead, fdLruTail, fd_next, fd_prev);
-	    ihP->ih_refcnt++;
-	    IH_UNLOCK;
-	    (void)FDH_SEEK(fdP, 0, SEEK_SET);
-	    return fdP;
 	}
+	ihP->ih_refcnt++;
+	IH_UNLOCK;
+	return fdP;
     }
 
     /*
@@ -374,7 +391,7 @@ ih_open_retry:
      * of open files reaches the size of the cache */
     if ((fdInUseCount > fdCacheSize || fd == INVALID_FD)  && fdLruHead != NULL) {
 	fdP = fdLruHead;
-	assert(fdP->fd_status == FD_HANDLE_OPEN);
+	osi_Assert(fdP->fd_status == FD_HANDLE_OPEN);
 	DLL_DELETE(fdP, fdLruHead, fdLruTail, fd_next, fd_prev);
 	DLL_DELETE(fdP, fdP->fd_ih->ih_fdhead, fdP->fd_ih->ih_fdtail,
 		   fd_ihnext, fd_ihprev);
@@ -394,7 +411,7 @@ ih_open_retry:
 	    fdHandleAllocateChunk();
 	}
 	fdP = fdAvailHead;
-	assert(fdP->fd_status == FD_HANDLE_AVAIL);
+	osi_Assert(fdP->fd_status == FD_HANDLE_AVAIL);
 	DLL_DELETE(fdP, fdAvailHead, fdAvailTail, fd_next, fd_prev);
 	closeFd = INVALID_FD;
     }
@@ -402,6 +419,7 @@ ih_open_retry:
     fdP->fd_status = FD_HANDLE_INUSE;
     fdP->fd_fd = fd;
     fdP->fd_ih = ihP;
+    fdP->fd_refcnt++;
 
     ihP->ih_refcnt++;
 
@@ -432,9 +450,10 @@ fd_close(FdHandle_t * fdP)
 	return 0;
 
     IH_LOCK;
-    assert(ih_Inited);
-    assert(fdInUseCount > 0);
-    assert(fdP->fd_status == FD_HANDLE_INUSE);
+    osi_Assert(ih_Inited);
+    osi_Assert(fdInUseCount > 0);
+    osi_Assert(fdP->fd_status == FD_HANDLE_INUSE ||
+               fdP->fd_status == FD_HANDLE_CLOSING);
 
     ihP = fdP->fd_ih;
 
@@ -443,14 +462,18 @@ fd_close(FdHandle_t * fdP)
      * failed (this is determined by checking the ihandle for the flag
      * IH_REALLY_CLOSED) or we have too many open files.
      */
-    if (ihP->ih_flags & IH_REALLY_CLOSED || fdInUseCount > fdCacheSize) {
+    if (fdP->fd_status == FD_HANDLE_CLOSING ||
+        ihP->ih_flags & IH_REALLY_CLOSED || fdInUseCount > fdCacheSize) {
 	IH_UNLOCK;
 	return fd_reallyclose(fdP);
     }
 
-    /* Put this descriptor back into the cache */
-    fdP->fd_status = FD_HANDLE_OPEN;
-    DLL_INSERT_TAIL(fdP, fdLruHead, fdLruTail, fd_next, fd_prev);
+    fdP->fd_refcnt--;
+    if (fdP->fd_refcnt == 0) {
+	/* Put this descriptor back into the cache */
+	fdP->fd_status = FD_HANDLE_OPEN;
+	DLL_INSERT_TAIL(fdP, fdLruHead, fdLruTail, fd_next, fd_prev);
+    }
 
     /* If this is not the only reference to the Inode then we can decrement
      * the reference count, otherwise we need to call ih_release.
@@ -480,19 +503,24 @@ fd_reallyclose(FdHandle_t * fdP)
 	return 0;
 
     IH_LOCK;
-    assert(ih_Inited);
-    assert(fdInUseCount > 0);
-    assert(fdP->fd_status == FD_HANDLE_INUSE);
+    osi_Assert(ih_Inited);
+    osi_Assert(fdInUseCount > 0);
+    osi_Assert(fdP->fd_status == FD_HANDLE_INUSE ||
+               fdP->fd_status == FD_HANDLE_CLOSING);
 
     ihP = fdP->fd_ih;
     closeFd = fdP->fd_fd;
+    fdP->fd_refcnt--;
 
-    DLL_DELETE(fdP, ihP->ih_fdhead, ihP->ih_fdtail, fd_ihnext, fd_ihprev);
-    DLL_INSERT_TAIL(fdP, fdAvailHead, fdAvailTail, fd_next, fd_prev);
+    if (fdP->fd_refcnt == 0) {
+	DLL_DELETE(fdP, ihP->ih_fdhead, ihP->ih_fdtail, fd_ihnext, fd_ihprev);
+	DLL_INSERT_TAIL(fdP, fdAvailHead, fdAvailTail, fd_next, fd_prev);
 
-    fdP->fd_status = FD_HANDLE_AVAIL;
-    fdP->fd_ih = NULL;
-    fdP->fd_fd = INVALID_FD;
+	fdP->fd_status = FD_HANDLE_AVAIL;
+	fdP->fd_refcnt = 0;
+	fdP->fd_ih = NULL;
+	fdP->fd_fd = INVALID_FD;
+    }
 
     /* All the file descriptor handles have been closed; reset
      * the IH_REALLY_CLOSED flag indicating that ih_reallyclose
@@ -502,10 +530,12 @@ fd_reallyclose(FdHandle_t * fdP)
 	ihP->ih_flags &= ~IH_REALLY_CLOSED;
     }
 
-    IH_UNLOCK;
-    OS_CLOSE(closeFd);
-    IH_LOCK;
-    fdInUseCount -= 1;
+    if (fdP->fd_refcnt == 0) {
+	IH_UNLOCK;
+	OS_CLOSE(closeFd);
+	IH_LOCK;
+	fdInUseCount -= 1;
+    }
 
     /* If this is not the only reference to the Inode then we can decrement
      * the reference count, otherwise we need to call ih_release. */
@@ -536,6 +566,7 @@ stream_fdopen(FD_t fd)
     streamP->str_fd = fd;
     streamP->str_buflen = 0;
     streamP->str_bufoff = 0;
+    streamP->str_fdoff = 0;
     streamP->str_error = 0;
     streamP->str_eof = 0;
     streamP->str_direction = STREAM_DIRECTION_NONE;
@@ -561,7 +592,7 @@ stream_open(const char *filename, const char *mode)
     } else if (strcmp(mode, "a+") == 0) {
 	fd = OS_OPEN(filename, O_RDWR | O_APPEND | O_CREAT, 0);
     } else {
-	assert(FALSE);		/* not implemented */
+	osi_Assert(FALSE);		/* not implemented */
     }
 
     if (fd == INVALID_FD) {
@@ -584,7 +615,7 @@ stream_read(void *ptr, afs_fsize_t size, afs_fsize_t nitems,
 	streamP->str_bufoff = 0;
 	streamP->str_buflen = 0;
     } else {
-	assert(streamP->str_direction == STREAM_DIRECTION_READ);
+	osi_Assert(streamP->str_direction == STREAM_DIRECTION_READ);
     }
 
     bytesRead = 0;
@@ -594,8 +625,8 @@ stream_read(void *ptr, afs_fsize_t size, afs_fsize_t nitems,
 	if (streamP->str_buflen == 0) {
 	    streamP->str_bufoff = 0;
 	    streamP->str_buflen =
-		OS_READ(streamP->str_fd, streamP->str_buffer,
-			STREAM_HANDLE_BUFSIZE);
+		OS_PREAD(streamP->str_fd, streamP->str_buffer,
+			STREAM_HANDLE_BUFSIZE, streamP->str_fdoff);
 	    if (streamP->str_buflen < 0) {
 		streamP->str_error = errno;
 		streamP->str_buflen = 0;
@@ -605,6 +636,7 @@ stream_read(void *ptr, afs_fsize_t size, afs_fsize_t nitems,
 		streamP->str_eof = 1;
 		break;
 	    }
+	    streamP->str_fdoff += streamP->str_buflen;
 	}
 
 	bytesToRead = nbytes;
@@ -637,7 +669,7 @@ stream_write(void *ptr, afs_fsize_t size, afs_fsize_t nitems,
 	streamP->str_bufoff = 0;
 	streamP->str_buflen = STREAM_HANDLE_BUFSIZE;
     } else {
-	assert(streamP->str_direction == STREAM_DIRECTION_WRITE);
+	osi_Assert(streamP->str_direction == STREAM_DIRECTION_WRITE);
     }
 
     nbytes = size * nitems;
@@ -645,13 +677,14 @@ stream_write(void *ptr, afs_fsize_t size, afs_fsize_t nitems,
     p = (char *)ptr;
     while (nbytes > 0) {
 	if (streamP->str_buflen == 0) {
-	    rc = OS_WRITE(streamP->str_fd, streamP->str_buffer,
-			  STREAM_HANDLE_BUFSIZE);
+	    rc = OS_PWRITE(streamP->str_fd, streamP->str_buffer,
+			  STREAM_HANDLE_BUFSIZE, streamP->str_fdoff);
 	    if (rc < 0) {
 		streamP->str_error = errno;
 		bytesWritten = 0;
 		break;
 	    }
+	    streamP->str_fdoff += rc;
 	    streamP->str_bufoff = 0;
 	    streamP->str_buflen = STREAM_HANDLE_BUFSIZE;
 	}
@@ -673,28 +706,25 @@ stream_write(void *ptr, afs_fsize_t size, afs_fsize_t nitems,
 
 /* fseek for buffered I/O handles */
 int
-stream_seek(StreamHandle_t * streamP, afs_foff_t offset, int whence)
+stream_aseek(StreamHandle_t * streamP, afs_foff_t offset)
 {
-    int rc;
+    ssize_t rc;
     int retval = 0;
 
     if (streamP->str_direction == STREAM_DIRECTION_WRITE
 	&& streamP->str_bufoff > 0) {
-	rc = OS_WRITE(streamP->str_fd, streamP->str_buffer,
-		      streamP->str_bufoff);
+	rc = OS_PWRITE(streamP->str_fd, streamP->str_buffer,
+		      streamP->str_bufoff, streamP->str_fdoff);
 	if (rc < 0) {
 	    streamP->str_error = errno;
 	    retval = -1;
 	}
     }
+    streamP->str_fdoff = offset;
     streamP->str_bufoff = 0;
     streamP->str_buflen = 0;
     streamP->str_eof = 0;
     streamP->str_direction = STREAM_DIRECTION_NONE;
-    if (OS_SEEK(streamP->str_fd, offset, whence) < 0) {
-	streamP->str_error = errno;
-	retval = -1;
-    }
     return retval;
 }
 
@@ -702,16 +732,18 @@ stream_seek(StreamHandle_t * streamP, afs_foff_t offset, int whence)
 int
 stream_flush(StreamHandle_t * streamP)
 {
-    int rc;
+    ssize_t rc;
     int retval = 0;
 
     if (streamP->str_direction == STREAM_DIRECTION_WRITE
 	&& streamP->str_bufoff > 0) {
-	rc = OS_WRITE(streamP->str_fd, streamP->str_buffer,
-		      streamP->str_bufoff);
+	rc = OS_PWRITE(streamP->str_fd, streamP->str_buffer,
+		      streamP->str_bufoff, streamP->str_fdoff);
 	if (rc < 0) {
 	    streamP->str_error = errno;
 	    retval = -1;
+	} else {
+	    streamP->str_fdoff += rc;
 	}
 	streamP->str_bufoff = 0;
 	streamP->str_buflen = STREAM_HANDLE_BUFSIZE;
@@ -724,16 +756,18 @@ stream_flush(StreamHandle_t * streamP)
 int
 stream_close(StreamHandle_t * streamP, int reallyClose)
 {
-    int rc;
+    ssize_t rc;
     int retval = 0;
 
-    assert(streamP != NULL);
+    osi_Assert(streamP != NULL);
     if (streamP->str_direction == STREAM_DIRECTION_WRITE
 	&& streamP->str_bufoff > 0) {
-	rc = OS_WRITE(streamP->str_fd, streamP->str_buffer,
-		      streamP->str_bufoff);
+	rc = OS_PWRITE(streamP->str_fd, streamP->str_buffer,
+		      streamP->str_bufoff, streamP->str_fdoff);
 	if (rc < 0) {
 	    retval = -1;
+	} else {
+	    streamP->str_fdoff += rc;
 	}
     }
     if (reallyClose) {
@@ -762,7 +796,7 @@ ih_fdclose(IHandle_t * ihP)
     int closeCount, closedAll;
     FdHandle_t *fdP, *head, *tail, *next;
 
-    assert(ihP->ih_refcnt > 0);
+    osi_Assert(ihP->ih_refcnt > 0);
 
     closedAll = 1;
     DLL_INIT_LIST(head, tail);
@@ -775,16 +809,22 @@ ih_fdclose(IHandle_t * ihP)
      */
     for (fdP = ihP->ih_fdhead; fdP != NULL; fdP = next) {
 	next = fdP->fd_ihnext;
-	assert(fdP->fd_ih == ihP);
-	assert(fdP->fd_status == FD_HANDLE_OPEN
-	       || fdP->fd_status == FD_HANDLE_INUSE);
+	osi_Assert(fdP->fd_ih == ihP);
+	osi_Assert(fdP->fd_status == FD_HANDLE_OPEN
+	       || fdP->fd_status == FD_HANDLE_INUSE
+	       || fdP->fd_status == FD_HANDLE_CLOSING);
 	if (fdP->fd_status == FD_HANDLE_OPEN) {
+	    /* Note that FdHandle_t's do not count against the parent
+	     * IHandle_t ref count when they are FD_HANDLE_OPEN. So, we don't
+	     * need to dec the parent IHandle_t ref count for each one we pull
+	     * off here. */
 	    DLL_DELETE(fdP, ihP->ih_fdhead, ihP->ih_fdtail, fd_ihnext,
 		       fd_ihprev);
 	    DLL_DELETE(fdP, fdLruHead, fdLruTail, fd_next, fd_prev);
 	    DLL_INSERT_TAIL(fdP, head, tail, fd_next, fd_prev);
 	} else {
 	    closedAll = 0;
+	    fdP->fd_status = FD_HANDLE_CLOSING;
 	    ihP->ih_flags |= IH_REALLY_CLOSED;
 	}
     }
@@ -793,9 +833,9 @@ ih_fdclose(IHandle_t * ihP)
      * closed all file descriptors.
      */
     if (ihP->ih_refcnt == 1 || closedAll) {
-	assert(closedAll);
-	assert(!ihP->ih_fdhead);
-	assert(!ihP->ih_fdtail);
+	osi_Assert(closedAll);
+	osi_Assert(!ihP->ih_fdhead);
+	osi_Assert(!ihP->ih_fdtail);
     }
 
     if (head == NULL) {
@@ -810,13 +850,14 @@ ih_fdclose(IHandle_t * ihP)
     for (fdP = head; fdP != NULL; fdP = fdP->fd_next) {
 	OS_CLOSE(fdP->fd_fd);
 	fdP->fd_status = FD_HANDLE_AVAIL;
+	fdP->fd_refcnt = 0;
 	fdP->fd_fd = INVALID_FD;
 	fdP->fd_ih = NULL;
 	closeCount++;
     }
 
     IH_LOCK;
-    assert(fdInUseCount >= closeCount);
+    osi_Assert(fdInUseCount >= closeCount);
     fdInUseCount -= closeCount;
 
     /*
@@ -845,19 +886,19 @@ ih_reallyclose(IHandle_t * ihP)
     ihP->ih_refcnt++;   /* must not disappear over unlock */
     if (ihP->ih_synced) {
 	FdHandle_t *fdP;
+        ihP->ih_synced = 0;
 	IH_UNLOCK;
-	
+
 	fdP = IH_OPEN(ihP);
-	if (fdP) { 
+	if (fdP) {
 	    OS_SYNC(fdP->fd_fd);
 	    FDH_CLOSE(fdP);
 	}
-	
+
 	IH_LOCK;
     }
 
-    assert(ihP->ih_refcnt > 0);
-    ihP->ih_synced = 0;
+    osi_Assert(ihP->ih_refcnt > 0);
 
     ih_fdclose(ihP);
 
@@ -883,7 +924,7 @@ ih_release(IHandle_t * ihP)
 	return 0;
 
     IH_LOCK;
-    assert(ihP->ih_refcnt > 0);
+    osi_Assert(ihP->ih_refcnt > 0);
 
     if (ihP->ih_refcnt > 1) {
 	ihP->ih_refcnt--;
@@ -938,17 +979,17 @@ ih_sync_all(void) {
 	if (ihP)
 	    ihP->ih_refcnt++;	/* must not disappear over unlock */
 	for (; ihP; ihP = ihPnext) {
-	    
+
 	    if (ihP->ih_synced) {
-		FdHandle_t *fdP;
+		FD_t fd;
 
 		ihP->ih_synced = 0;
 		IH_UNLOCK;
 
-		fdP = IH_OPEN(ihP);
-		if (fdP) { 
-		    OS_SYNC(fdP->fd_fd);
-		    FDH_CLOSE(fdP);
+		fd = OS_IOPEN(ihP);
+		if (fd != INVALID_FD) {
+		    OS_SYNC(fd);
+		    OS_CLOSE(fd);
 		}
 
 	  	IH_LOCK;
@@ -983,9 +1024,6 @@ ih_sync_thread(void *dummy) {
 	IOMGR_Sleep(60);
 #endif /* AFS_PTHREAD_ENV */
 
-#ifndef AFS_NT40_ENV
-        sync();
-#endif
         ih_sync_all();
     }
     return NULL;
@@ -1014,14 +1052,40 @@ ih_icreate(IHandle_t * ih, int dev, char *part, Inode nI, int p1, int p2,
 }
 #endif /* AFS_NAMEI_ENV */
 
-
-#ifndef AFS_NT40_ENV
 afs_sfsize_t
-ih_size(int fd)
+ih_size(FD_t fd)
 {
+#ifdef AFS_NT40_ENV
+    LARGE_INTEGER size;
+    if (!GetFileSizeEx(fd, &size))
+	return -1;
+    return size.QuadPart;
+#else
     struct afs_stat status;
     if (afs_fstat(fd, &status) < 0)
 	return -1;
     return status.st_size;
-}
 #endif
+}
+
+#ifndef HAVE_PIO
+ssize_t
+ih_pread(int fd, void * buf, size_t count, afs_foff_t offset)
+{
+	afs_foff_t code;
+	code = OS_SEEK(fd, offset, 0);
+	if (code < 0)
+	    return code;
+	return OS_READ(fd, buf, count);
+}
+
+ssize_t
+ih_pwrite(int fd, const void * buf, size_t count, afs_foff_t offset)
+{
+	afs_foff_t code;
+	code = OS_SEEK(fd, offset, 0);
+	if (code < 0)
+	    return code;
+	return OS_WRITE(fd, buf, count);
+}
+#endif /* !HAVE_PIO */
